#include "RT_HW_Wire.h"


/****************************************************************************************
                           ИНТЕРФЕЙС i2c
****************************************************************************************/
#define WIRE_PERIOD_FIND_MIN  100	//--Минимальное время поиска устройства
#define WIRE_CNT_FIND_MAX     20	//--Минимальное время поиска устройства
//****************************************************************************************
// 				КОНСТРУКТОР  КЛАССА  RT_HW_WIRE	
//****************************************************************************************
RT_HW_WIRE:: RT_HW_WIRE(){
	init=false;
	codeDevice=0;		//--код контроллера
	Freq =400;  //--частота шины i2с
	Limit=230;	//--параметр для библиотеки ESP8266
	SDA  =RT_HW_WIRE_SDA;	SCL  =RT_HW_WIRE_SCL; //--номера пинов для основной шины i2c
	SDA1 =-1; 				SCL1 =-1;			 //--номера пинов для дополнительной шины i2c	
#ifdef	RT_HW_WIRE_SDA1		//--Переустановка
	SDA1=RT_HW_WIRE_SDA1;	//        параметра 
#endif						//				 SDA1
#ifdef	RT_HW_WIRE_SCL1		//--Переустановка
	SCL1 =RT_HW_WIRE_SCL1;	//        параметра 
#endif						//				 SCL1
};

//****************************************************************************************
// 				ФУНКЦИИ  КЛАССА RT_HW_WIRE	
//****************************************************************************************
//---------------------------------------------------------------------------------------
//                           Инициализация шины i2c
//  Инициализирует шину с защитой от повторной инициализации (initWire=true)
//---------------------------------------------------------------------------------------
void RT_HW_WIRE:: begin(int16_t vSDA, int16_t vSCL, int16_t vFreq, int16_t vLimit){
  if(init) return;				//--Выход, если шина i2c инициализирована
  init=true;	 				//--Установка флага "шина i2c инициализирована"
  if(vSDA>=0)	SDA =vSDA; 		//--Установка передаваемого в функции значения SDA
  if(vSCL>=0)	SCL  =vSCL; 
  if(vFreq>=0)	Freq=vFreq;	
  if(vLimit>=0)	Limit=vLimit; 
//------Инициализации шига под тип контроллера  
#if   RT_HW_WIRE_DEFINED_AVR328			//--Контроллеры AVR328      
	Wire.begin(); Wire.setClock(Freq*(uint32_t)1000); codeDevice=RT_HW_WIRE_CODE_AVR328;	
  #elif RT_HW_WIRE_DEFINED_MEGA2560		//--Контроллеры MEGA2560
	Wire.begin(); Wire.setClock(Freq*(uint32_t)1000); codeDevice=RT_HW_WIRE_CODE_MEGA2560;	
  #elif RT_HW_WIRE_DEFINED_SAM3X8E  	//--Контроллеры SAM3X8E
    Wire.begin();  									  codeDevice=RT_HW_WIRE_CODE_SAM3X8E;
	//Wire.setClock(RT_HW_Board.i2c_freq*(uint32_t)1000);	
  #elif RT_HW_WIRE_DEFINED_ESP8266	    //--Контроллеры ESP8266				
	Wire.pins(SDA, SCL);  
	Wire.begin(); Wire.setClock(Freq*(uint32_t)1000); codeDevice=RT_HW_WIRE_CODE_ESP8266;		
  #elif RT_HW_WIRE_DEFINED_ESP32  	    //--Контроллеры ESP32				        
	Wire.begin(SDA, SCL,(Freq*(uint32_t)1000)); 	  codeDevice=RT_HW_WIRE_CODE_ESP32;		
  #elif RT_HW_WIRE_DEFINED_STM32		//--Контроллеры STM32
	Wire.begin();	
	Wire.setClock(Freq*(uint32_t)1000);								  
	codeDevice=RT_HW_WIRE_CODE_STM32;	
  #else     							//--Контроллер ANON 
#endif
  //Serial.println(" begin code="+ String(codeDevice));
};	//end begin()

//---------------------------------------------------------------------------------------
//       ПОСЫЛКА  АДРЕСА НА I2C ДЛЯ ПОЛУЧЕНИЯ ОТКЛИКА ОТ УСТРОЙСТВА С ЭТИМ АДРЕСОМ 
//---------------------------------------------------------------------------------------
uint8_t RT_HW_WIRE:: write	(int16_t addr){						
		Wire.beginTransmission(addr); 		//--Начинает передачу на ведомое i2c устройство
return	Wire.endTransmission();}		//--Завершение передачи в i2c
//---------------------------------------------------------------------------------------
//                          ЗАПИСЬ БАЙТА В I2C
//---------------------------------------------------------------------------------------
uint8_t	RT_HW_WIRE:: write	(int16_t addr, uint8_t val){						
		Wire.beginTransmission(addr); 			//--Начинает передачу на ведомое i2c устройство
		Wire.write(val); 							//--Запись данных в устройство
return	Wire.endTransmission();}				//--Завершение передачи в i2c
//---------------------------------------------------------------------------------------
//                          ЗАПИСЬ МАССИВА В I2C
//---------------------------------------------------------------------------------------

uint8_t	RT_HW_WIRE:: write	(int16_t addr, uint8_t *data, uint8_t length){
		Wire.beginTransmission(addr); 		//--Начинает передачу на ведомое i2c устройство
		Wire.write(data,length); 				//--Запись данных в устройство
return	Wire.endTransmission();}			//--Завершение передачи в i2c	
//---------------------------------------------------------------------------------------
//                          ЗАПИСЬ СТРОКИ В I2C
//---------------------------------------------------------------------------------------
uint8_t RT_HW_WIRE:: write	(int16_t addr, char mess[]){
		Wire.beginTransmission(addr); 		//--Начинает передачу на ведомое i2c устройство
		Wire.write(mess); 						//--Запись данных в устройство
return	Wire.endTransmission();}			//--Завершение передачи в i2c
//---------------------------------------------------------------------------------------
//                          ЧТЕНИЕ БАЙТОВ ИЗ I2C
//---------------------------------------------------------------------------------------
//uint8_t RT_HW_WIRE:: read(){};
//---------------------------------------------------------------------------------------
//                           Сканирование шины i2c 
//---------------------------------------------------------------------------------------
int16_t  RT_HW_WIRE:: scan(String &S, uint8_t time){int16_t qnt;
   qnt=0;   
   for (int i=1;i < 128; i++)
   { Wire.beginTransmission(i);  
   if (Wire.endTransmission()==0) {S=S+String(F(" ")) + String(i,HEX) + String(F("/")) + String(i,DEC) + String(F(";")); qnt++;}
   delay(time);
   }
   S=S+String(F(" Qnt=")) + String(qnt);
return qnt;}	

/*
//---------------------------------------------------------------------------------------
//                          ПОИСК УСТРОЙСТВА НА
//---------------------------------------------------------------------------------------
bool RT_HW_WIRE:: find	(RT_HW_WIRE_DEVICE &z, uint8_t divide ){
	if(z.findSlave) 	return 1;			//--Выход, если устройство уже обнаружено
	switch(z.findStep){default:z.findStep=0;
	//---Предварительная проверка возможности поиска устройства
	case 0:	if(z.blockSlave)	return 1;	//--Выход, если устройство заблокировано (устанавливает пользователь)
			if(!z.paramSlave)	return 0;	//--Если параметры устройства для шины i2c  не инициализированы, то выход
			if(!init)         	return 0;	//--Если шина i2c не инициализирована, то выход
			z.findStep++; 					//--Переход
	case 1: z.codeErr=send(z.addr);			//--Посылка адреса для получения отклика от устройства
			if (z.codeErr==0)  {z.findSlave=true; z.findStep=0; return 1;}	//--Выход, если устройство обнаружено 
		    if((z.findCnt < divide&0x2F) && (z.findCnt < 50)) z.findCnt++;	//--Подготовка к следующей посылке  
			z.period=WIRE_PERIOD_FIND_MIN * z.findCnt;   					//		через увеличивающийся интервал
			z.timePeriod=millis(); z.findStep++; return 0; //   
	case 2: if(getPastTense(z.timePeriod,millis()) > z.period) {z.findStep=1;} return 0; //--Пауза перед следующим поиском
}};	
*/

//---------------------------------------------------------------------------------------
//         ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ
//---------------------------------------------------------------------------------------
//uint32_t RT_HW_WIRE:: getPastTense (uint32_t &t1, uint32_t t2){if (t2>=t1) {return (t2-t1);} else {return (0xFFFFFFFF-t1+t2);}};    
//---------------------------------------------------------------------------------------
//                           СОЗДАНИЕ ОБЪЕКТА 
//--------------------------------------------------------------------------------------- 
RT_HW_WIRE RT_HW_Wire;
//=================================================================================================