/*** FLProg ************** Библиотека RT_HW_Main.h ver 2.1 для Arduino ***************
  Библиотеки серии RT состоит из различных библиотек, построенных по общим приницпам,
  для контролеров AVR, MEGA2560, SAM3XDUE, ESP8266, ESP32, STM32

  
  Библиотеки  разрабатываются с возможностью самостоятельного использования 
  и с учетом использования в системе визуального программирования FLProg.
  ecoins@mail.ru
  30.03.2019г.
**************************************************************************************************/
//#################################################################################################
#ifndef RT_HW_Wire_h
#define RT_HW_Wire_h

#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_WIRE_VERSION    11
//---Макросы идентификации плат
#define RT_HW_WIRE_DEFINED_AVR328   defined (__AVR_ATmega328__) || defined (__AVR_ATmega328P__) || defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__)
#define RT_HW_WIRE_DEFINED_MEGA2560 defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__) 
#define RT_HW_WIRE_DEFINED_SAM3X8E  defined(__SAM3X8E__)
#define RT_HW_WIRE_DEFINED_ESP8266  defined(ESP8266) 
#define RT_HW_WIRE_DEFINED_ESP32    defined(ESP32) 
#define RT_HW_WIRE_DEFINED_STM32    defined(_VARIANT_ARDUINO_STM32_) 
#define RT_HW_WIRE_DEFINED_ANON  
//++++Подключение библиотеки
#include <Wire.h>					
//+++++++++++++++++ ВЫБОР ИНДИВИДУАЛЬНЫХ ПАРАМЕТРОВ ДЛЯ КОНТРОЛЛЕРОВ+++++++++++
//----Определение кодов плат для использования в библиотеках  и внешних программах 
enum { RT_HW_WIRE_CODE_NO=0,      RT_HW_WIRE_CODE_AVR328, RT_HW_WIRE_CODE_MEGA2560, RT_HW_WIRE_CODE_SAM3X8E, 
       RT_HW_WIRE_CODE_ESP8266,   RT_HW_WIRE_CODE_ESP32,  RT_HW_WIRE_CODE_STM32,    RT_HW_WIRE_CODE_ANON};
//----------------- AVR ATmega328,328P,168,88 ---------------------------------
#if         RT_HW_WIRE_DEFINED_AVR328
	#define RT_HW_WIRE_CODE_BOARD	RT_HW_CODE_AVR328
	#define RT_HW_WIRE_SDA			17
	#define RT_HW_WIRE_SCL		   	18
//----------------- AVR ATmega2560,256 -----------------------------------------	
  #elif     RT_HW_WIRE_DEFINED_MEGA2560 
	#define RT_HW_WIRE_SDA			20
	#define RT_HW_WIRE_SCL		   	21
//----------------- SAM32X8E DUE ----------------------------------------------
  #elif     RT_HW_WIRE_DEFINED_SAM3X8E
	#define RT_HW_WIRE_SDA			20
	#define RT_HW_WIRE_SCL		   	21
	#define RT_HW_WIRE_SDA1			-1
	#define RT_HW_WIRE_SCL1		   	-1	
//--------------- ESP8266 ----------------------------------------------------
  #elif     RT_HW_WIRE_DEFINED_ESP8266
	#define RT_HW_WIRE_SDA			 4
	#define RT_HW_WIRE_SCL		   	 5
//----------------- ESP32 ------------------------------------------------------
  #elif 	RT_HW_WIRE_DEFINED_ESP32	
	#define RT_HW_WIRE_SDA			21
	#define RT_HW_WIRE_SCL		   	22
//----------------- STM32 ------------------------------------------------------
  #elif   RT_HW_WIRE_DEFINED_STM32                	
	#define RT_HW_WIRE_SDA			23	//--соотвествует SCL1
	#define RT_HW_WIRE_SCL		   	22	//--соотвествует SCL2
	#define RT_HW_WIRE_SDA1			27	//--соотвествует SDA2
	#define RT_HW_WIRE_SCL1		   	26	//--соотвествует SCL2		
//----------------- NO_TYPE_CONTROLLER -----------------------------------------
  #else 
	#define RT_HW_WIRE_SDA			-1
	#define RT_HW_WIRE_SCL		   	-1
#endif 
//****************************************************************************************
// 							 RT_HW_WIRE	
//				 класс функций для управления устройствами на шине i2c
//****************************************************************************************
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_WIRE{
public:
//----Данные
uint8_t init:1, codeErr:3;
int16_t SDA,SCL,SDA1,SCL1;
int16_t Freq; int16_t Limit;
uint8_t codeDevice;
//---Методы
RT_HW_WIRE();                //--конструктор
void 	begin	(int16_t sda=-1, int16_t scl=-1, int16_t freq=400, int16_t limit=230); //--Инициализация шины i2c
uint8_t write	(int16_t addr);									//--Обращение к устройству без записи данных
uint8_t write	(int16_t addr, uint8_t val);					//--Запись байта в i2c
uint8_t write	(int16_t addr, uint8_t *data, uint8_t length);	//--Запись массива байтов в i2c
uint8_t write	(int16_t addr, char mess[]);							//--Запись строки   в i2c
//uint8_t read	();												//--Чтение массива из i2c
int16_t scan(String &S, uint8_t time=1);              // time - интервал опроса шины i2c в мсек
//uint32_t getPastTense (uint32_t &t1, uint32_t t2);
};
#pragma pack(pop)				//---Восстановление предыдущей настройки
//============Глобальные ссылки на созданные объекты
extern RT_HW_WIRE		RT_HW_Wire;		//----Глобальная ссылка на созданный объект RT_HW_Wire
//#################################################################################################
#endif //END #ifndef RT_HW_Main_h
