#include "RT_HW_MCP23017.h"
//-------------------------------------------------------------------------------------------------
//          Параметры - адреса для IOCON.BANK=0
//-------------------------------------------------------------------------------------------------
#define MCP_BASE	0x20	//регистр направления пинов A: =0x00(default) - все на выход, 0xff - все на вход
#define MCP_IODIRA	0x00	//регистр направления пинов A: =0x00(default) - все на выход, 0xff - все на вход
#define MCP_IODIRB	0x01	//регистр направления пинов B: =0x00(default) - все на выход, 0xff - все на вход
#define MCP_IPOLA	0x02	//регистр инверсии    пинов A: =0x00(default) - все прямые,   0xff - все инверсные
#define MCP_IPOLB	0x03	//регистр инверсии    выводов регистра B: =0x00(default) - все прямые,   0xff - все инверсные
#define MCP_GPPUA	0x0C	//регистр подтяжки входов регистра A резисторами 100кОм: =0x00(default) - не подтянуты,   0xff - все подтянуты
#define MCP_GPPUB	0x0D	//регистр подтяжки входов регистра A резисторами 100кОм: =0x00(default) - не подтянуты,   0xff - все подтянуты
#define MCP_GPIOA	0x12	//регистр выводов  на пины A
#define MCP_GPIOB	0x13	//регистр выводов  на пины B
#define MCP_IOCON   0x0A	//регистр конфигурации
//-------------------------------------------------------------------------------------------------
//          Параметры для функций
//-------------------------------------------------------------------------------------------------
#define MCP_PERIOD_FIND_MIN  100 //--Минимальный период повторного поиска устройства на шине i2c 
#define MCP_PERIOD_DEVICE_MIN 20 //--Минимальный период вызова в мс
#define MCP_QNT_ERROR_MAX	   3 //--Максимальное кол-во ошибок при одном выполнении обмена с устройством 
#define MCP_DELAY_FOR_ERROR   10 //--Пауза между повторными транзакциями при ошибке обмена
#define MCP_MAX_TIME_READ      2 //--Максимальное время на считывание регистров
#define MCP_DIVIDER_FIND	  20 //--Делитель при поиске
#define MCP_PERIOD_DEFINE	 250 //--Период по умолчанию
//-------------------------------------------------------------------------------------------------
//          RT_HW_MCP23017_ID:: КОНСТРУКТОР
//-------------------------------------------------------------------------------------------------
RT_HW_MCP23017_ID:: RT_HW_MCP23017_ID(){init=device=false; };
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
//                           ИНИЦИАЛИЗАЦИЯ ПАРАМЕТРОВ УСТРОЙСТВА
//uint8_t  dirA:1, dirB:1, ipolA:1,ipolB:1, gppuA:1, gppuB:1;    
// dirA,dirB: =0 -вывод, =1 - ввод без инверсии и подтяжки; 3 -ввод с инверсией без подтяжки; 
//			  =5 -ввод без инверсии с подтяжкой; 7 -ввод с инверсией и подтяжкой; остальные значения -- ввод без инверсии и подтяжки;
//-------------------------------------------------------------------------------------------------
bool RT_HW_MCP23017:: setID(RT_HW_MCP23017_ID &id, int16_t addr, uint8_t dirA, uint8_t dirB){
	if(id.init)	return 1; id.init=true;					//--Выход, если устройство уже инициализировано. 
	Wire.begin();										//--Если i2c уже инициирован, то вызов функции никаких действй не вызывает.
	id.addr=-1; id.block=true;  						//--Предварительная установка флага блокировки устройства. 
	if(addr <0)				  {id.addr=-1; 				 id.block=true; }//--Если addr<1, обмен будет заблокирован.
	if((addr>=0) && (addr<=7)){id.addr=MCP_BASE+addr; 	 id.block=false;}//--Если адрес от 0-7, то это адрес в адресном пространстве MCP_BASE
	if(addr>=MCP_BASE)		  {id.addr=addr;			 id.block=false;}//--Если адрес по фактической адресации на шине i2c 
	id.divider= MCP_DIVIDER_FIND;						 //--Делитель периода при поиске устройства
	id.findStep=id.beginStep=id.workStep=id.deviceStep=0;//--Очистка флагов
	id.outA=id.outB=id.agoOutA=id.agoOutB=id.inA=id.inB=0;//--Очистка буферов 	
	id.link=id.device=id.noFind=false;		//--флаги "отсутствия связи" и "устройство не найдено"
	id.run=1;
	switch(dirA){	//----НАСТРОЙКА РЕГИСТРА А
	case 0:		id.dirA=false;	id.ipolA=false;	id.gppuA=false;	break;	//--Вывод.
	case 1:		id.dirA=true; 	id.ipolA=false; id.gppuA=false;	break;	//--Ввод без инверсии  и без подтяжки.
	case 3:		id.dirA=true; 	id.ipolA=true;	id.gppuA=false;	break;	//--Ввод  c  инверсией и без подтяжки.
	case 5:		id.dirA=true; 	id.ipolA=false; id.gppuA=true;	break;	//--Ввод без инверсии  и  с  подтяжкой.	
	case 7:		id.dirA=true; 	id.ipolA=true;	id.gppuA=true;	break;	//--Ввод  c  инверсией и  с  подтяжкой.	
	default:	id.dirA=true;	id.ipolA=false;	id.gppuA=false;	break;}	//--Ввод без инверсии и без подтяжки.
	switch(dirB){	//----НАСТРОЙКА РЕГИСТРА В
	case 0:		id.dirB=false;	id.ipolB=false;	id.gppuB=false;	break;	//--Вывод.
	case 1:		id.dirB=true; 	id.ipolB=false; id.gppuB=false;	break;	//--Ввод без инверсии  и без подтяжки.
	case 3:		id.dirB=true; 	id.ipolB=true;	id.gppuB=false;	break;	//--Ввод  c  инверсией и без подтяжки.
	case 5:		id.dirB=true; 	id.ipolB=false; id.gppuB=true;	break;	//--Ввод без инверсии  и  с  подтяжкой.	
	case 7:		id.dirB=true; 	id.ipolB=true;	id.gppuB=true;	break;	//--Ввод  c  инверсией и  с  подтяжкой.	
	default:	id.dirB=true;	id.ipolB=false;	id.gppuB=false;	break;}	//--Ввод без инверсии и без подтяжки.
return 1;};
//-------------------------------------------------------------------------------------------------
//            			ПОИСК MCP23017 (выход 
//-------------------------------------------------------------------------------------------------
bool RT_HW_MCP23017:: find(RT_HW_MCP23017_ID &id) {
switch(id.findStep){ 
    default: id.findStep=0;
	//----ИНИЦИАЛИЗАЦИЯ ПАРАМЕТРОВ
	case 0:	id.findCnt=1; id.findStep++; //--Установка счетчика делителя периода поиска устройства и переход на следующий этап.
	//----ПОИСК УСТРОЙСТВА НА ШИНЕ шине i2c
	case 1: Wire.beginTransmission(id.addr); 		//--Начинает передачу на ведомое i2c устройство
			id.codeErr=Wire.endTransmission(); 		//--Завершение передачи в i2c	
			if (id.codeErr==0) {id.findStep=0; id.device=1; id.link=1; id.noFind=0; return 1;} id.findStep++;
	//----ПОДГОТОВКА К ПОВТОРНОМУ ПОИСКУ ЧЕРЕЗ УВЕЛИЧИВАЮЩУЮСЯ ПАУЗУ
	case 2: id.timePeriod=millis(); if(id.findCnt < id.divider) id.findCnt*=2; 
			if(id.findCnt>id.divider) {id.findCnt = id.divider; id.device=0; id.link=0; id.noFind=1; id.findStep=1; return 1;} id.findStep++;
	//----ПАУЗА ДЛЯ СЛЕДУЩЕГО ЦИКЛА ПОИСКА
	case 3: if(getPastTense(id.timePeriod,millis()) > MCP_PERIOD_FIND_MIN * id.findCnt) {id.findStep=1;} return 0; //--Пауза перед следующим поиском
} return 0;};	
//-------------------------------------------------------------------------------------------------
//            			 ИНИЦИАЛИЗАЦИЯ УСТРОЙСТВА
// dirA:1, dirB:1, ipolA:1,ipolB:1, gppuA:1, gppuB:1;   
//-------------------------------------------------------------------------------------------------
bool RT_HW_MCP23017:: begin(RT_HW_MCP23017_ID &id){
if(id.device)		return 1;						//--Выход, если устройство не инициализировано
switch(id.beginStep){								//--ПЕРЕКЛЮЧАТЕЛЬ ЭТАПОВ ФУНКЦИИ
	default: id.beginStep=0;						//--Обработка не определенного этапа
	//----ПРОВЕРКА УСЛОВИЙ ДЛЯ ЗАПУСКА ФУНКЦИИ ИНИЦИАЛИЗАЦИИ
	case 0:	if( id.block)				  return 1;	//--Выход, если устройство заблокировано (устанавливает пользователь)
			if(!id.init)	{id.device=0; return 0;}//--Если параметры устройства не инициализированы, устанавливается не готовность устройства и выход	
			id.beginStep++; 						//--Переход на следующий этап
			//Serial.println("begin_0 link=" + String(id.link) + " device=" + String(id.device));				
	//----ПОИСК УСТРОЙСТВА НА ШИНЕ i2c 
	case 1:  find(id);
			if(id.noFind){id.beginStep=0; return 0;}//--Если взведен флаг "Устройство не найдено", прекращение инициализации и переход на начало.
			if(!id.link)				 {return 0;}//--Если связь не установлена, выходим до следующего вхождения в функцию. 	 
			id.beginStep++;							//--Связь установлена и переход на следующий этап.	
	//----ИНИЦИАЛИЗАЦИЯ НАПРАВЛЕНИЯ И ПОЛЯРНОСТИ(для ввода) РЕГИСТРОВ 
	case 2: id.data[0x00]=MCP_IODIRA; 							//--Адрес регистра, с которого будут записаны четыре параметра.
			if(id.dirA)	 id.data[1]=0xff;	else id.data[1]=0;	//--Установка направление регистра A (ввод/вывод). 
			if(id.dirB)  id.data[2]=0xff;	else id.data[2]=0;	//--Установка направление регистра В (ввод/вывод).
			if(id.ipolA) id.data[3]=0xff;	else id.data[3]=0;	//--Установка полярности регистров A (при работк на ввод).  
			if(id.ipolB) id.data[4]=0xff;	else id.data[4]=0;	//--Установка полярности регистров A (при работк на ввод).  	
			Wire.beginTransmission(id.addr); 					//--Начинает передачу на ведомое i2c устройство.
			Wire.write(id.data,5); 								//--Запись посылки из 5 байт.
			id.codeErr=Wire.endTransmission(); 					//--Завершение передачи в i2c.	
			if (id.codeErr!=0) {id.device=id.link=false; id.beginStep=0; return 1;} //--Выход, если ошибка обмена с i2c.
			id.beginStep++;										//--Если нормально, переход на следующий этап.
	//----ИНИЦИАЛИЗАЦИЯ ПОДТЯЖКИ РЕГИСТРОВ ВНУТРЕННИМИ РЕЗИСТОРАМИ 100к(для ввода)
	case 3: id.data[0x00]=MCP_GPPUA; 							//--Адрес регистра, с которого будут записаны четыре параметра.
			if(id.gppuA) id.data[1]=0xff;	else id.data[1]=0;	//--Установка направление регистра A (ввод/вывод). 
			if(id.gppuB) id.data[2]=0xff;	else id.data[2]=0;	//--Установка направление регистра В (ввод/вывод).
			Wire.beginTransmission(id.addr); 					//--Начинает передачу на ведомое i2c устройство
			Wire.write(id.data,3); 								//--Запись посылки из трех байт
			id.codeErr=Wire.endTransmission(); 					//--Завершение передачи в i2c	
			if (id.codeErr!=0) {id.device=id.link=false; id.beginStep=0; return 1;} //--Выход, если ошибка обмена с i2c.
			id.device=true; id.beginStep=0;id.cntErr=0;return 1;//--Если нормально, взводится флаг готовности устройства и завершение функции.
} return 0;};		
//-------------------------------------------------------------------------------------------------
//            			 ОБМЕН С УСТРОЙСТВОМ
//uint8_t  dirA:1, dirB:1, ipolA:1,ipolB:1, gppuA:1, gppuB:1;    
//-------------------------------------------------------------------------------------------------
bool RT_HW_MCP23017:: device(RT_HW_MCP23017_ID &id){
switch(id.deviceStep){//--ПЕРЕКЛЮЧАТЕЛЬ ЭТАПОВ ФУНКЦИИ
	default: id.deviceStep=0;							//--Обработка не определенного этапа
	//----ПРОВЕРКА ВНЕШНЕГО ФЛАГА ЗАПУСКА РАБОТЫ С УСТРОЙСТВОМ ДЛЯ НАЧАЛА ОБМЕНА С УСТРОЙСТВОМ 
	case 0: if (!id.run)   return 1; id.run=false;	id.deviceStep++; //--Выход из флаг запуск=0, иначе сброс этого флага и продолжение работы.
	//----ПРОВЕРКА ГОТОВНОСТИ УСТРОЙСТВА 
	//    Если устройство не готово (id.device==0), то оно заново инициализируется функцией begin(id). 
	//    Функция begin() осуществляет щадящий поиск устройства на шине i2c; 
	//	  Если инициализация устройства не удается, тогда нормальный выход.
	case 1: if(!id.device) {if(!begin(id))   return 0;} //--Инициализация 
			if(!id.device) {id.deviceStep=0; return 1;}	 id.deviceStep++;
	//----ЗАПИСЬ В УСТРОЙСТВО
	case 2: if(!write(id)) return 0; id.workStep=0; id.deviceStep++;
	//----ЧТЕНИЕ ИЗ УСТРОЙСТВА 
	case 3:	if(!read(id))  return 0; id.workStep=0; id.deviceStep++;
	//----ЗАВЕРШЕНИЕ ОБМЕНА С УСТРОЙСТВОМ И ВЫХОД
	case 4: id.deviceStep=0; return 1;
	}
return 0;}; 							//--Нормальный выход	
//-------------------------------------------------------------------------------------------------
//                           ЗАПИСЬ В РЕГИСТРЫ A,B (одновременный вызов с read() не допускается
//-------------------------------------------------------------------------------------------------
bool RT_HW_MCP23017:: write(RT_HW_MCP23017_ID &id){	
switch(id.workStep){default: id.workStep=0;
	//----ПРОВЕРКА УСЛОВИЙ ДЛЯ НАЧАЛА ОБМЕНА С УСТРОЙСТВОМ (Если устройство не готово, нормальный выход)
	case 0: if(!id.device) return 1; id.workStep++;	  
	//----ПРОВЕРКА НА НЕОБХОДИМОСТЬ ВЫВОДА (если оба регистра настроены на ввод, то нормальный выход). 
	//----НАСТРОЙКА ТРАНЗАКЦИИ (первый адрес записываемого регистра, значение регистра А, значение регитсра В ) 
	case 1: if((id.dirA==1) && (id.dirB==1)) {id.workStep=0; return 1;}  id.cntErr=0; 
			if((id.dirA==0) && (id.dirB==1)) {id.data[0]=MCP_GPIOA; id.data[1]=id.outA;                     id.qntData=1;}  
			if((id.dirA==1) && (id.dirB==0)) {id.data[0]=MCP_GPIOB; 					id.data[1]=id.outB; id.qntData=1;}  
			if((id.dirA==0) && (id.dirB==0)) {id.data[0]=MCP_GPIOA; id.data[1]=id.outA; id.data[2]=id.outB; id.qntData=2;}  
			id.workStep++;  
	//----КОНТРОЛЬ НА ОГРАНИЧЕНИЕ ОШИБОК (если больше, сброс готовности устройства, готовности связи и нормальный выход=1).
	case 2: if(id.cntErr>MCP_QNT_ERROR_MAX) {id.workStep=0; id.device=id.link=false;    return 1;} id.workStep++;	
	//----ЗАПИСЬ В УСТРОЙСТВО ТРАНЗАКЦИИ (если ошибка при передаче, передача управление на паузу и временный выход)	 
	case 3:	Wire.beginTransmission(id.addr); 					//--Начинает передачу на ведомое i2c устройство
			Wire.write(id.data,(id.qntData+1)); 						//--Запись посылки из трех байт
			id.codeErr=Wire.endTransmission(); 					//--Завершение передачи в i2c	
	       //id.codeErr=RT_HW_Wire.write(id.addr,id.data,id.qntData+1);			  				//--Запись транзакции
			if(id.codeErr!=0) {id.cntErr++; id.timeWork=millis(); id.workStep=7;        return 0;}	//--Если ошибка, то временный выход с переходом на паузу
			id.workStep=0;															    return 1;  	//--Нормальный выход по завершению		 
	//----ПАУЗА ПЕРЕД ПОВТОРНОЙ ПОПЫТКОЙ ТРАНЗАКЦИИ
	case 7: if(getPastTense(id.timeWork,millis()) >MCP_DELAY_FOR_ERROR){id.workStep=2;} return 0;	//--Пауза перед следующей попыткой и временный выход
}
return 1;} 						
//-------------------------------------------------------------------------------------------------
//                           ЧТЕНИЕ В РЕГИСТРЫ A,B
//-------------------------------------------------------------------------------------------------
bool RT_HW_MCP23017:: read(RT_HW_MCP23017_ID &id){	
switch(id.workStep){default: id.workStep=0;
	//----ПРОВЕРКА УСЛОВИЙ ДЛЯ НАЧАЛА ОБМЕНА С УСТРОЙСТВОМ (Если устройство не готово, нормальный выход)
	case 0: if(!id.device) return 1; id.cntErr=0; id.workStep++;	  
	//----ПРОВЕРКА НА НЕОБХОДИМОСТЬ ВВОДА (если оба регистра настроены на вывод, inA=inB=0 и нормальный выход). 
	case 1: if((id.dirA==0) && (id.dirB==0)){id.inA=0; id.inB=0; id.workStep=0; return 1;}  id.workStep++; 
	//----КОНТРОЛЬ НА ОГРАНИЧЕНИЕ ОШИБОК (если больше, сброс готовности устройства, готовности связи и нормальный выход=1).
	case 2: if(id.cntErr>MCP_QNT_ERROR_MAX) {id.workStep=0; id.device=id.link=false; return 1;} id.workStep++;	
	//----ОТПРАВКА ЗАПРОСА НА ЧТЕНИЕ ДВУХ РЕГИСТРОВ С АДРЕСА MCP_GPIOA
	case 3:	Wire.beginTransmission(id.addr); 		//--Начинает передачу на ведомое i2c устройство
			Wire.write(MCP_GPIOA); 					//--Запись адреса регистра в устройство
			id.codeErr=Wire.endTransmission(); 		//--Завершение передачи в i2c	
			Wire.requestFrom(id.addr,2); 			//--Запрос на чтение двух регистров
	        id.timeWork=millis(); id.workStep++;	//--Установка таймер и переход на следующий этап
	//-----ОЖИДАНИЕ ПРИХОДА ДВУХ БАЙТОВ С ТАЙМ_АУТОМ
	case 4: do{if(getPastTense(id.timeWork,millis()) > MCP_MAX_TIME_READ) {id.codeErr=7; break;}} while(Wire.available() < 2);
			if(id.codeErr !=0) {id.timeWork=millis(); id.cntErr++; id.workStep=7; return 0;}
			id.workStep++;	
	//-----ЧТЕНИЕ РЕГИСТРОВ А И И
	case 5:	id.inA=Wire.read(); if(id.dirA==0) id.inA=0;
			id.inB=Wire.read(); if(id.dirB==0) id.inB=0;
			id.workStep=0; return 1;				//--Нормальный выход	
	//----ПАУЗА ПЕРЕД ПОВТОРНОЙ ПОПЫТКОЙ ТРАНЗАКЦИИ ПОСЛЕ ОШИБКИ
	case 7: if(getPastTense(id.timeWork,millis()) > MCP_DELAY_FOR_ERROR) {id.workStep=2;} return 0; //--Пауза перед следующей попыткой и временный выход
}
return 1;} 		
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//---------------------------------------------------------------------------------------
//         ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ
//---------------------------------------------------------------------------------------
uint32_t  RT_HW_MCP23017:: getPastTense (uint32_t &t1, uint32_t t2){if (t2>=t1) {return (t2-t1);} else {return (0xFFFFFFFF-t1+t2);}};  
//---------------------------------------------------------------------------------------
//            ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
//---------------------------------------------------------------------------------------
 bool     RT_HW_MCP23017:: eventMillis (uint32_t &t1,uint32_t val){if(getPastTense (t1,millis())>val) {t1+=val; return 1;} return 0;};
 //---------------------------------------------------------------------------------------
//  ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ (более точно и не зависимо чем event)  
//---------------------------------------------------------------------------------------
bool 	  RT_HW_MCP23017:: periodMillis(uint8_t &s,uint32_t &t1,uint32_t v){if(s==0){t1=millis();s=1;}if(getPastTense(t1,millis())>v){t1=millis();return 1;}return 0;} 
//---------------------------------------------------------------------------------------
//                   УСТАНОВКА ПЕРИОДА ОБНОВЛЕНИЯ ДАННЫХ  
//--------------------------------------------------------------------------------------- 
void 	RT_HW_MCP23017:: setPeriod(RT_HW_MCP23017_ID &id, int32_t extPeriod) { 
if(extPeriod  <0) id.period=-1; 
if(extPeriod  =0) id.period=250; 
if(extPeriod  >0) id.period=extPeriod; 
if((id.period >0) && (id.period < MCP_PERIOD_DEVICE_MIN)) id.period=MCP_PERIOD_DEVICE_MIN;}    
//---------------------------------------------------------------------------------------		
RT_HW_MCP23017 RT_HW_mcp23017;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++