/**************************************************************************************************
 *       БИБЛИОТЕКА ДЛЯ РАСШИРИТЕЛЯ ВВОДА/ВЫВОДА
 *					mcp23017   
 ***********************************************************
  Написана ecoins (ecoins@mail.ru) 
   2019-01-21: Начальная версия
   2018-08-12: Оптимизированая под работу  с FLProg
   2019-06-10: Полностью изменен подход
   2019_07_18: Добавлено чтение входов
   2019_07_30: Полностью переписана
 ******************************************************************/
#ifndef RT_HW_MCP23017_h
#define RT_HW_MCP23017_h

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
#include "RT_HW_Wire.h"			//--Подключение библиотеки
#define RT_HW_MCP23017_VER 14	//--НОМЕР ВЕРСИИ
//****************************************************************************************
//				RT_HW_MCP23017_ID	класс данных для устройства mcp23017
//****************************************************************************************
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_MCP23017_ID{
public:
int16_t  addr;
uint8_t  init:1, device:1, agoDevice:1, link:1, noFind:1, qntData:3;
uint8_t  dirA:1, dirB:1, ipolA:1,  ipolB:1, gppuA:1, gppuB:1, run:1, block:1;    
uint8_t	 outA, outB, agoOutA, agoOutB, inA, inB, agoInA, agoInB;	
uint32_t timePeriod, timeWork;		//--рабочий регистр поиска устройства
int16_t  period;					//--текущий период поиска устройства
uint8_t  data[5];
uint8_t  divider,findCnt;
uint8_t  codeErr;
uint16_t findStep:3, beginStep:3, workStep:3, deviceStep:3, cntErr:3;
// ****** Методы ******
RT_HW_MCP23017_ID();    	//--Конструктор
};
#pragma pack(pop)			//---Восстановление предыдущей настройки
//++++++++++++++++++++Описание класса DHT_RT++++++++++++++++++++++++++++
//****************************************************************************************
// 				RT_HW_WIRE_DEVICE_ID класс данных для устройства на шине i2c
//****************************************************************************************
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_MCP23017{
public:
// ****** Данные ******
// ****** Методы ******
RT_HW_MCP23017(){}; 							//--Конструктор
bool    setID		(RT_HW_MCP23017_ID &id, int16_t addr, uint8_t dirA=1, uint8_t dirB=1);	//--Установка параметров устройства  
bool	find		(RT_HW_MCP23017_ID &id);	//--Поиск устройства на шине i2c                              
bool    begin 		(RT_HW_MCP23017_ID &id);	//--Инициализация устройства
bool    device		(RT_HW_MCP23017_ID &id); 	//--Обмен с устройством
bool    write		(RT_HW_MCP23017_ID &id); 	//--Вывод на пины значения регистров id.GPIOA, idGPIOB;
bool    read		(RT_HW_MCP23017_ID &id);	//--Чтение из устройтсва
void 	setPeriod	(RT_HW_MCP23017_ID &id, int32_t extPeriod); 
bool	getNoFind	(RT_HW_MCP23017_ID &id){return id.noFind;};	//--Возвращает флаг - устройство не обнаружено
bool	getLink		(RT_HW_MCP23017_ID &id){return id.link;};	//--Возвращает флаг - связь с устройством  установлена	
bool	getDevice	(RT_HW_MCP23017_ID &id){return id.link;};	//--Возвращает флаг - устройство готово к работе 	
bool	getInitID	(RT_HW_MCP23017_ID &id){return id.init;};	//--Возвращает флаг - параметры устройства инициализированы
bool 	checkOutA	(RT_HW_MCP23017_ID &id){if(id.agoOutA==id.outA) return 0; id.agoOutA=id.outA; return 1;};
bool 	checkOutB	(RT_HW_MCP23017_ID &id){if(id.agoOutB==id.outB) return 0; id.agoOutB=id.outB; return 1;};
bool 	checkInA	(RT_HW_MCP23017_ID &id){if(id.agoInA ==id.inA)  return 0; id.agoInA =id.inA;  return 1;};
bool 	checkInB	(RT_HW_MCP23017_ID &id){if(id.agoInB ==id.inB)  return 0; id.agoInB =id.inB;  return 1;};
bool 	checkDevice	(RT_HW_MCP23017_ID &id){if(id.agoDevice ==id.device)  return 0; id.agoDevice =id.device;  return 1;};
//--------------------------------------------------------------------------------------------------
uint32_t getPastTense	(uint32_t &t1, uint32_t t2);		//--ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ 
bool 	 eventMillis	(uint32_t &t1, uint32_t val);	    //--ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
bool 	 periodMillis   (uint8_t &s,   uint32_t &t1, uint32_t v);
};  //---END 
#pragma pack(pop)				//---Восстановление предыдущей настройки
//-------------------------------------------------------------
//==============  Конец класса ======================================================
extern RT_HW_MCP23017 RT_HW_mcp23017; //--Создание объекта LCD типа HD44780
//===================================================================================
#endif //---RT_HW_MCP23017_h


