
//**********************************************
// Перечень основных функций:   
//**********************************************
#include "RT_HW_LCD_HD44780.h"
//--КОДЫ КОМАНД НАСТРОЙКИ LCD------------------------------------------------------- 
enum{BACKLIGHT_ON	=0x08,	//включение подсветкой	Lcd (исполняется после любого вывода на Lcd)
	 BACKLIGHT_OFF	=0x00,	//выключение подсветкой	Lcd (исполняется после любого вывода на Lcd)
	 CLEAR_DISPLAY	=0x01,	//очистка дисплея (время 1,5мс)
	 RETURN_HOME	=0x02,	//перемещает курсор в левую позицию  
	 ENTRY_SET		=0x04,	//направ.сдвига курсора и разреш.сдвига дисплея при выводе данных на Lcd 
	 CONTROL_SET	=0x08,	//команда включения курсора и его типа (вкл. дисплея здесб не используется)
	 SHIFT_SET		=0x10,	//команда сдвига дисплея или курсора 
	 FUNCTION_SET	=0x20,	//команда модификации дисплея
	 SET_CGRAM_ADDR	=0x40,	//команда установка адреса Flash  для последующих операций
	 SET_DDRAM_ADDR	=0x80};	//команда установка адреса данных (RAM) для последующих операций
//--ФЛАГИ ДЛЯ КОМАНДЫ ENTRY_MODE_SET------------------------------------------------	 
enum{//--ФЛАГИ ДЛЯ КОМАНДЫ ------------------------------------------------
	 ENTRY_CURSOR_RIGHT=0x00,	ENTRY_CURSOR_LEFT	=0x02,	//направление сдвига курсора вправо/влево
	 ENTRY_DISPLAY_ON  =0x01,	ENTRY_DISPLAY_OFF	=0x00, 	//разрешения сдвига дисплея
	 //--ФЛАГИ ДЛЯ КОМАНДЫ ДЛЯ DISPLAY_CONTROL-------------------------------------------
	 CONTROL_DISPLAY_ON=0x04,	CONTROL_DISPLAY_OFF	=0x00,	//включение/выключение дисплея
	 CONTROL_CURSOR_ON =0x02,	CONTROL_CURSOR_OFF	=0x00,	//включение/выключения курсора
     CONTROL_BLINK_ON  =0x01,	CONTROL_BLINK_OFF	=0x00,	//мерцающий блок/подчеркивание
	 //--ФЛАГИ ДЛЯ КОМАНДЫ ДЛЯ SHIFT_CURSOR_DISPLAY---------------------------------------
     SHIFT_DISPLAY_MOVE=0x08,	SHIFT_CURSOR_MOVE	=0x00,	//сдвиг дисплея или курсора
     SHIFT_MOVE_RIGHT  =0x04,	SHIFT_MOVE_LEFT		=0x00,	//направление сдвига(дисплея или курсора) 
     //--ФЛАГИ ДЛЯ КОМАНДЫ ДЛЯ FUNCTION_SET (набор функций)-------------------------------
 	 FUNCTION_MODE8	   =0x10,	FUNCTION_MODE4		=0x00,	//8-ми или 4-х разрядный интерфейс
	 FUNCTION_LINE2    =0x08,	FUNCTION_LINE1		=0x00,	//кол-во линий в дисплее
     FUNCTION_DOTS5x10 =0x04,	FUNCTION_DOTS5x8	=0x00,	//размер символа
     //--ФЛАГИ ДЛЯ ОБМЕНА ТЕТРАДАМИ
     EN=B00000100, RW=B00000010, RS=B00000001};	
//--КОНСТАНТЫ
enum{CMND=0, DATA=1};

#define LCD_MODE_RT									1	//--Режим работы со временем: =1 -реальное время; =0
#define LCD_HOT_RESTART_PERIOD_DEFINE			60000	//--(мс) Период горячего рестарта по умолчанию
#define LCD_HOT_RESTART_PERIOD_MIN		         1000	//--(мс) Период горячего рестарта минимальный (для отладки)
#define LCD_DATA_PERIOD_DEFINE	  		          500	//--(мс) Период выдачи данных для DatatID по умолчанию
#define LCD_DATA_PERIOD_MIN	  		               50	//--(мс) Период выдачи данных для DatatID минимальный


#define LCD_DELAY_RESTART_DEVICE				  500   //--(мс) Пауза  попытками рестарта устройства, если при нормальной работе возник сбой связи
#define LCD_DELAY_RESTART_QNT				        6   //--(мс) Кол-во попыток рестарта устройства, если при нормальной работе возник сбой связи
#define LCD_DELAY_NO_DEVICE                      5000	//--(мс) Паузы между поиском устройства, если устройства нет или связь с ним устойчиво потеряна
#define LCD_DELAY_BEGIN_CONTROL_DEVICE	 	       10  	//--(мс) Пауза после ошибки вывода символа			
#define RT_HW_LCD_HD44780_DELAY_BETWEEN_VALUE      10	//--(мс) Паузы между выводами чисел 
#define RT_HW_LCD_HD44780_DELAY_BETWEEN_STRING     50	//--(мс) Паузы между выводами строк 

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//----------------------------------------------------------------------------------------
//						КОНСТРУКТОРЫ 
//----------------------------------------------------------------------------------------
RT_HW_LCD_HD44780_DeviceID:: RT_HW_LCD_HD44780_DeviceID(){init=false; periodHot=60 * 1000UL;};
RT_HW_LCD_HD44780_DataID::	 RT_HW_LCD_HD44780_DataID()	 {init=false; period=500;};	
RT_HW_LCD_HD44780:: 		 RT_HW_LCD_HD44780() 		 {initDevice=0;}
//---------------------------------------------------------------------------------------
// 			              ***УСТАНОВКА ID LCD 
// Вход:  1)проверяет допустимость номера устройства (
// Выход: 1)Взведенный бит инициализации устройства в регистре initDevice
//		  2)Указатель на ID устройства в массиве указателей prt[num]	  
//---------------------------------------------------------------------------------------
bool RT_HW_LCD_HD44780:: setDeviceID (uint8_t num, RT_HW_LCD_HD44780_DeviceID  &z, char interface, uint8_t cols,uint8_t rows, uint8_t addr){
  if(z.init) return 1; 
  if(num >= RT_HW_LCD_HD44780_DEVICE_MAX)	return 0;	//--Выход, если номер устройства превышает допустимый
  prt[num]=&z;											//--Запоминание указателя ID
  z.num=num;											//--Запоминание номера устройства
 // bitSet(initDevice,num);								//--Установка бита инициализации устройства
  z.link=z.busy=z.out=false;   				    //--Сброс флагов
  z.errOk=z.errCode=z.errCnt=0;							//--Сброс параметров ошибок связи
  z.Step={0,0,0,0,0,0};									//--Сброс шагов функций
  z.cols=constrain(cols,1,RT_HW_LCD_HD44780_MAX_COLS);  //--Кол-во колонок LCD 
  z.rows=constrain(cols,1,RT_HW_LCD_HD44780_MAX_ROWS);  //--Кол-во   строк LCD
  z.interface=interface;								//--Запоминание типа интерфейса (I -i2c,SPI -spi,P -7 pins)
  z.cntDevice=0; z.device=0;
  z.delayBusy=0;
  switch(z.interface){
	  default:
	  case 'N':  			 z.init=false; break;
	  case 'I': z.addr=addr; z.init=true;  z.delayBusy=3; break;
	  case 'S': z.addr=addr; z.init=true; break;
	  case 'P': z.addr=addr; z.init=true; z.Rs=14; z.Es=15; z.DB4=4; z.DB5=5; z.DB6=6; z.DB7=7; break;
	  }
	for(int i=0; i<z.cols; i++) {z.buff[i]=' ';}  						//--Очистка буфера вывода
//--Установка бита успешной инициализации устройства	
  bitSet(initDevice,z.num);												//--Установка бита успешной инициализации устройства	
  z.rt=LCD_MODE_RT;														//--Установка выбранного режима реального времени	
  z.periodNoDevice=LCD_DELAY_RESTART_DEVICE; z.periodNoDevice=millis();	//--Устанавливаем параметры времени поиска устройства
  z.periodHot=LCD_HOT_RESTART_PERIOD_DEFINE; z.timePeriodHot=millis();		//--Устанавливаем параметры времени горячего перезапуска
  z.dividerWork=0; z.divider=10;							//--
  z.run=z.init=true; 													//--Устанавливаем флаги запуска и инициализации 
return 1;};
//----------------------------------------------------------------------------------------
//					*****УСТАНОВКА ID СООБЩЕНИЯ 
// uint32_t lcd:4, row:4, col:6, len:6, index:6, step:4;    uint8_t  id:1, init:1, blink:1, fresh:1, point:2;
// char alig; char sign; uint32_t time; float val; String s;
// RUN - внешний сигнал разрешения инициализации. Инициализация происходит только пока x.init=false;
//----------------------------------------------------------------------------------------  
bool RT_HW_LCD_HD44780:: setDataID (uint8_t num, RT_HW_LCD_HD44780_DataID  &id, uint8_t col, uint8_t row, uint8_t len, char alig, uint8_t point) {
	if(id.init) 			             return 1; //--Выход без изменения параметров инцициализации, если устройство уже инициализировано
	if(num>RT_HW_LCD_HD44780_DEVICE_MAX) return 0; //--Выход без инициализации, если номер устройства больше разрешенного 
	if(!bitRead(initDevice,num))	     return 0; //--Выход без инициализации, если устройство с указанным номером не инициализировано
	id.lcd=num; id.row=row;	  id.col=col;	   id.alig=alig;   id.len=len; id.point=point & 0x03; //--Запоминаем входные параметры
	id.s=" *";  
	//id.val=NAN;	  
	id.blink=0;	   id.step=0;	    //--Устанавливаем параметры рабочие параметры
	id.valInt16=0; id.valChar=' '; id.valFloat=NAN; 
	id.period=LCD_DATA_PERIOD_DEFINE;   id.timePeriod=millis();	//--Устанавливаем параметры времени
	id.blockRepeat=false;										//--Флаг блокировки повторяемого вывода (например для фиксированного текста)
	id.run=id.init=true; 					                    //--Устанавливаем флаги запуска и инициализации 
return 1;};  
//---------------------------------------------------------------------------------------
// 			        ****КОНТРОЛЬ И ИНИЦИАЛИЗАЦИЯ ДИСПЛЕЯ 
//----п.0 Проверка условий начала инициализации. п.1-3: Двухкратное с временным промежутком (для надежности) обращение к Lcd. 
//---------------------------------------------------------------------------------------
bool RT_HW_LCD_HD44780:: begin(RT_HW_LCD_HD44780_DeviceID &z) {if(z.rt) BEGIN(z); else {do {} while (!BEGIN(z));}}; 
bool RT_HW_LCD_HD44780:: BEGIN(RT_HW_LCD_HD44780_DeviceID &z) {uint32_t x;
//--------------------------------------------------------------------------------	
  switch(z.Step.begin) {default: z.Step.begin=0;	
  case 0: if(!z.device) { if(periodMillis(z.s_timeNoDevice, z.timeNoDevice, z.periodNoDevice)) 
                          { if(z.cntDevice>LCD_DELAY_RESTART_QNT) {z.periodNoDevice=LCD_DELAY_NO_DEVICE;} else {z.cntDevice++; z.periodNoDevice=LCD_DELAY_RESTART_DEVICE;} 
                            z.device=1;} return 1;} //--Если устройство не обнаружено ->выход
          if( !z.init)             	return 1; //Выход,  если устройство не инициализировано	
		  if((!z.run) && z.link)  return 1;
		  z.run=z.link=z.errOk=0; z.busy=z.out=1; z.Step={0,0,0,0,0,0,0,0}; z.errCode=z.errCnt=0; z.errOk=0; z.Step.begin++; return 0; //Инициализация параметров и переход на следующий этап	
  case 1: z.buffSend[0]=BACKLIGHT_ON; writeBuff(z,1); 
          if(z.errCode==0){z.timeStartDelay=millis(); z.Step.begin++; return 0;} //Устройство    отвечает -> переход на следующий этап
		              else{z.busy=z.out=z.device=0;   z.Step.begin=0; return 1;} //Устройство не отвечает -> выход
  case 2: if(getPastTense(z.timeStartDelay,millis()) > LCD_DELAY_BEGIN_CONTROL_DEVICE) {z.Step.begin++;} return 0; //Переход по окончание задержки по времени 
  case 3: z.buffSend[0]=BACKLIGHT_ON; writeBuff(z,1);  
          if(z.errCode==0){z.cntDevice=0;             z.Step.begin++; return 0;} //Устройство    отвечает -> переход на следующий этап
		             else {z.busy=z.out=z.device=0;   z.Step.begin=0; return 1;} //Устройство не отвечает -> выход
  case 4: if(initBus4bit(z)) 																					{z.Step.begin++;} return 0; //Настройка на 4-х битной шине (в соответствии с datasheet HD44780)
  case 5: x=FUNCTION_SET | FUNCTION_MODE4     | FUNCTION_LINE2     | FUNCTION_DOTS5x8;  if(send(z,x,CMND,2000))	{z.Step.begin++;} return 0; //Обмен по 4-х разрядной шине, 2-линии, символ 5х8.
  case 6: x=SHIFT_SET    | SHIFT_CURSOR_MOVE  | SHIFT_MOVE_LEFT;                        if(send(z,x,CMND,2000)) {z.Step.begin++;} return 0; //Сдвиг курсора, перемещения курсора влево
  case 7: x=CONTROL_SET	 | CONTROL_DISPLAY_ON | CONTROL_CURSOR_OFF | CONTROL_BLINK_OFF; if(send(z,x,CMND,2000)) {z.Step.begin++;} return 0; //Включение дисплея, выключение курсора, тип курсора "мерцание"
  case 8: x=ENTRY_SET    | ENTRY_CURSOR_LEFT  | ENTRY_DISPLAY_OFF;                      if(send(z,x,CMND,2000)) {z.Step.begin++;} return 0; //Движение курсора влево, выключение дисплея
  case 9: x=CLEAR_DISPLAY;                                                              if(send(z,x,CMND,2000)) {z.Step.begin++;} return 0; //Очистка дисплея
  case 10:z.link=true; z.busy=z.out=false; 										       				             z.Step.begin=0;  return 1; //Завершение настройки		
 } return 1;} 
//---------------------------------------------------------------------------------------
//					****ВЫВОД СТРОКИ
//----------------------------------------------------------------------------------------
bool RT_HW_LCD_HD44780:: print (RT_HW_LCD_HD44780_DataID  &x) {if(prt[x.lcd]->rt) PRINT (x); else {do {} while (!PRINT (x));}};
bool RT_HW_LCD_HD44780:: PRINT (RT_HW_LCD_HD44780_DataID  &x){
	//return 1;
//===========================================================
if ((!RT_HW_Lcd_hd44780.prt[x.lcd]->link)) 	{x.run=1; x.step=0;	return 1;}	//---Проверка и выход, если нет связи с дисплеем. При этоим устанавливаем флаг готовности
if (!x.run)  														return 1;	//---Проверка и выход если нет свежих данных
if (!x.init)                               	  						return 1; 	//---Проверка и выход, если параметры сообщения не инициализированы
//if (!bitRead(RT_HW_Lcd_hd44780.initDevice,x.lcd))				    return 1;
//============================================================
//---Автомат управления выводом 
  switch (x.step){default:  										x.step=0;	return 0; //return 0;	//--Обработка неопределенности
    //---Проверка правильности курсора
	case 0: if ((x.col>= RT_HW_Lcd_hd44780.prt[x.lcd]->cols) || (x.row>= RT_HW_Lcd_hd44780.prt[x.lcd]->rows)) {x.run=0; x.ok=1; break;} //return 1;} 
			x.step++; 
    //---***** ПРОВЕРКА ЗАНЯТОСТИ УСТРОЙСТВА. Если занято - то ожидание.  
	case 1: if(RT_HW_Lcd_hd44780.prt[x.lcd]->busy)               {return 0;}   
	        else {RT_HW_Lcd_hd44780.prt[x.lcd]->busy=1; x.step++; return 0;} //return 0;
	//---**** ВЫЧИСЛЕНИЕ ПАРАМЕТРОВ СТРОКИ 
	case 2: x.lenStr=x.s.length(); 								    //--Длина входной строки
	        x.lenMax=RT_HW_Lcd_hd44780.prt[x.lcd]->cols - x.col;	//--Максимально допустимая длина с учетом положение куросора и длины строки устройства
			if((x.len==0) || (x.len>x.lenMax)) x.lenOut=x.lenMax;	//--Вычисление 
			                              else x.lenOut=x.len;		//    длины выходного буфера
			if(x.lenStr > x.lenOut)    x.lenStr=x.lenOut;			//       и ограничение длины входной строки.
	 //---**** ПЕРЕХОД НА ВЫВОД СТРОКИ ИЛИ НА ВЫВОД СИМВОЛА ИЗ СТРОКИ (режим мерцания) (blink=1)
			if(!x.blink) x.step+=2; else x.step++; return 0;; //return 0;		
	//--- ***** ПОДГОТОВКА ВЫВОДА ОДНОГО СИМВОЛА (режим мерцание - один символ циклически выбирается из входной строки)
	case 3: RT_HW_Lcd_hd44780.prt[x.lcd]->buff[0]=x.s.charAt(x.lenBlank);//--Циклический выбор символа  
	        x.lenBlank++;  if(x.lenBlank>=x.lenStr) x.lenBlank=0;   //       из входной строки
			RT_HW_Lcd_hd44780.prt[x.lcd]->out=1;	x.index=0;	x.lenOut=1;						   x.step=5; return 0;;//return 0; //--Переход на вывод		
	//--- ***** ПОДГОТОВКА ВЫВОДА СТРОКИ
    case 4: x.lenBlank=0; if(x.lenOut > x.lenStr){  			    //---Установка кол-во пробелов 
				if(x.alig=='C') {x.lenBlank=(x.lenOut-x.lenStr)/2; }	// 		в начале вывода для выравнивания
				if(x.alig=='R') {x.lenBlank=(x.lenOut-x.lenStr);} }	//                       при alig='C' или 'R'
			//---Очищаем пробелами буфер и заполняем его выводимым сообщением 
			for(int i=0; i < x.lenOut; i++) {RT_HW_Lcd_hd44780.prt[x.lcd]->buff[i]           =' ';};	
	        for(int i=0; i < x.lenStr; i++) {RT_HW_Lcd_hd44780.prt[x.lcd]->buff[x.lenBlank+i]=x.s.charAt(i);}  
			//---Переход на вывод в Lcd
			RT_HW_Lcd_hd44780.prt[x.lcd]->out=1;	x.index=0;								x.step++; return 0;; ///return 0; 
	//--- ***** ВЫВОД В УСТРОЙСТОВО 
	case 5: if(send(*RT_HW_Lcd_hd44780.prt[x.lcd], pointCursor(x.col,x.row),                 CMND,50)){           x.step++;} return 0; //return 0; //--установка курсора
	case 6: if(send(*RT_HW_Lcd_hd44780.prt[x.lcd],RT_HW_Lcd_hd44780.prt[x.lcd]->buff[x.index],DATA,0)){x.index++; x.step++;} return 0;;//return 0; //--вывод данных
	case 7: if(x.index < x.lenOut) 	{x.step--;}	else {x.step++;}                				  							 return 0;//return 0; //     в устройство	
    //--- **** ОБРАБОТКА ЗАВЕРШЕНИЯ ВЫВОДА 	  
	case 8: RT_HW_Lcd_hd44780.prt[x.lcd]->out=0;  x.step++; return 0;
	case 9: RT_HW_Lcd_hd44780.prt[x.lcd]->busy=0; x.run=0; 							  x.step=0;	return 1;;
} 
return 1;
}; //--END 	
//---------------------------------------------------------------------------------------
//                **** ВОЗВРАЩАЕТ АДРЕС ПОЗИЦИИ ПО НОМЕРУ КОЛОНКИ И НОМЕРУ СТРОКИ 
//--------------------------------------------------------------------------------------- 
uint8_t  RT_HW_LCD_HD44780:: pointCursor(uint8_t col, uint8_t row) {uint8_t val;	
	//col= constrain(col,0,(z.cols-1));  row= constrain(row,0,(z.rows-1));  // ?????
	switch(row){
		 case 0: val=0x00; break;
		 case 1: val=0x40; break;
		 case 2: val=0x14; break;
		 case 3: val=0x54; break;}
return (0x80 | (col + val));}   
//---------------------------------------------------------------------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//---------------------------------------------------------------------------------------
// 			   **** ПЕРЕВОД ДИСПЛЕЯ В РЕЖИМ ИНИЦИАЛИЗАЦИИ ДЛЯ 4-Х БИТНОГО ОБМЕНА 
//			          see datasheet HD44780U (LCD-II) HITACHI Fig.24
//---------------------------------------------------------------------------------------
bool RT_HW_LCD_HD44780:: initBus4bit(RT_HW_LCD_HD44780_DeviceID &z){	
  switch(z.Step.initBus4){  
    default:													z.Step.initBus4=0;
	case 0: z.timeStartDelay=micros();							z.Step.initBus4++;	
	case 1: if(getPastTense(z.timeStartDelay,micros())>5000)	z.Step.initBus4++; return 0;//--Задержка(мкс) для установления питания													 										
	case 2: if(send4(z, (B0011 << 4), 4500))					z.Step.initBus4++; return 0;
 	case 3: if(send4(z, (B0011 << 4), 4500))					z.Step.initBus4++; return 0;
	case 4: if(send4(z, (B0011 << 4), 4500))					z.Step.initBus4++; return 0;	
	case 5: if(send4(z, (B0010 << 4), 2000))					z.Step.initBus4++; return 0;
    //-----------------------------------------------------------------------------------	
	case 6: if(send(z, B00100000,CMND,2000))					z.Step.initBus4++; return 0;	 //--NF	
	case 7: if(send(z, B00000001,CMND,2000))					z.Step.initBus4++; return 0;
	case 8: if(send(z, B00000100,CMND,2000))					z.Step.initBus4++; return 0;
    case 9:														z.Step.initBus4=0; return 1;	
return 1;}};
//---------------------------------------------------------------------------------------
//            ***ТЕТРАДЫ В LCD  (БИТЫ  4-7 VALUE - ОТПРАВЛЯЕМАЯ ТЕТРАДА)  
//---------------------------------------------------------------------------------------
bool RT_HW_LCD_HD44780:: send4(RT_HW_LCD_HD44780_DeviceID &z, uint8_t value, uint32_t time){uint8_t v;
switch(z.Step.send4){default: z.Step.send4=0;
	 case  0: v=value;
	          z.buffSend[0] = v       | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
			  z.buffSend[1] = v | EN  | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
			  z.buffSend[2] = v       | BACKLIGHT_ON;	 
			  z.buffSend[3] =(v & ~EN)| BACKLIGHT_ON;	//--Выделение младшей тетрады	   			  
		      z.Step.send4++; return 0;
     case 1:  if(writeBuff(z,4)) 		                z.Step.send4++; return 0;  //--Вывод буфера в устройство
	 case 2:  if (z.errCode!=0) 		{z.link=false; 	z.Step.send4=0; return 1;} //--Выход, если при обмене была ошибка
			  if(time==0)				{				z.Step.send4=0; return 1;} //--Выход, если после вывода не нужна пауза
			  z.timeStartDelay=micros();                z.Step.send4++; return 0;  //--Подготовка задержки и переход на следующий этап
     case 3:  if(getPastTense(z.timeStartDelay,micros())>time) {z.Step.send4=0; return 1;}  return 0;  //--Отработка паузы между передачи байта (если time !=0)   
} return 1;};	
//-----------------------------------------------------------------------------------
//		      ****ОТПРАВКА БАЙТА В LCD  ---- 
//---------------------------------------------------------------------------------------
bool RT_HW_LCD_HD44780:: send (RT_HW_LCD_HD44780_DeviceID &z, uint8_t value, uint8_t mode, uint32_t time){uint8_t v;
switch(z.Step.send){
default: z.Step.send=0;
	 case  0:
     	v=(value & 0xf0) | mode;						//--Выделение старшей тетрады								
			z.buffSend[0] = v       | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
			z.buffSend[1] = v | EN  | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
			z.buffSend[2] = v       | BACKLIGHT_ON;	 
			z.buffSend[3] =(v & ~EN)| BACKLIGHT_ON;	//--Выделение младшей тетрады	   	
	    v=(value<<4 & 0xf0)| mode;											
			z.buffSend[4] = v       | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
			z.buffSend[5] = v |  EN | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
			z.buffSend[6] = v       | BACKLIGHT_ON; 
			z.buffSend[7] =(v & ~EN)| BACKLIGHT_ON;			  
		z.Step.send++; return 0;
     case 1: if(writeBuff(z,8))							z.Step.send++; return 0;  //--Вывод буфера в устройство
	 case 2: if(z.errCode!=0)			{z.link=false;  z.Step.send=0; return 0;} //--Выход, если при обмене была ошибка
	         if(time==0)      			{				z.Step.send=0; return 1;} //--Выход, если после вывода не нужна пауза
			 z.timeStartDelay=micros();  				z.Step.send++; return 0;  //--Подготовка задержки и переход на следующий этап
     case 3: if(getPastTense(z.timeStartDelay,micros())>time) {z.Step.send=0;  return 1;} return 0;  //--Отработка паузы между передачи байта (если time !=0)
}return 1;};	 
//---------------------------------------------------------------------------------------
//   		  ****ЗАПИСЬ БУФЕРА В УСТРОЙСТВО В LCD (для i2c c контролем ошибок)  
//---------------------------------------------------------------------------------------	
bool RT_HW_LCD_HD44780:: writeBuff(RT_HW_LCD_HD44780_DeviceID &z, uint8_t len){ 
 switch(z.interface){
	 default:
	 case 'N': z.errOk=true; break; //--
//-------------PINS------------------------------------------------------------
	 case 'P': z.errOk=true; break; 
//-------------I2C-------------------------------------------------------------
	 case 'I':         
			 Wire.beginTransmission(z.addr); 
             Wire.write(z.buffSend,len);
			     z.errCode=Wire.endTransmission();
			 if (z.errCode!=0) z.errOk=true;		 
			 break;
//------------SPI--------------------------------------------------------------			 
	 case 'S':   z.errOk=true; break;		 
	 
 }
	 return 1;}; 
//---------------------------------------------------------------------------------------
//         ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ
//---------------------------------------------------------------------------------------
uint32_t RT_HW_LCD_HD44780:: getPastTense (uint32_t &t1, uint32_t t2){if (t2>=t1) {return (t2-t1);} else {return (0xFFFFFFFF-t1+t2);}};  
//---------------------------------------------------------------------------------------
//            ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
//---------------------------------------------------------------------------------------
bool     RT_HW_LCD_HD44780:: eventMillis (uint32_t &t1,uint32_t val){if(getPastTense (t1,millis())>val) {t1+=val; return 1;} return 0;};
//---------------------------------------------------------------------------------------
//  ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ (более точно и не зависимо чем event)  
//---------------------------------------------------------------------------------------
bool 	 RT_HW_LCD_HD44780:: periodMillis(uint8_t &s,uint32_t &t1,uint32_t v){if(s==0){t1=millis();s=1;}if(getPastTense(t1,millis())>v){t1=millis();return 1;}return 0;}
//---------------------------------------------------------------------------------------
//                   УСТАНОВКА ПЕРИОДА ГОРЯЧЕГО ПЕРЕЗАПУСКА  
//--------------------------------------------------------------------------------------- 
void 	RT_HW_LCD_HD44780:: setPeriodHot(RT_HW_LCD_HD44780_DeviceID &id, int32_t extPeriod) {
id.periodHot=LCD_HOT_RESTART_PERIOD_DEFINE; 
if(extPeriod >0) id.periodHot=extPeriod; 
if(id.periodHot < LCD_HOT_RESTART_PERIOD_MIN) id.periodHot=LCD_HOT_RESTART_PERIOD_MIN;     }	
//---------------------------------------------------------------------------------------
//                   УСТАНОВКА ПЕРИОДА ОБНОВЛЕНИЯ ДАННЫХ  
//--------------------------------------------------------------------------------------- 
void 	RT_HW_LCD_HD44780:: setPeriod(RT_HW_LCD_HD44780_DataID &id, int32_t extPeriod) {
id.period=LCD_DATA_PERIOD_DEFINE; 
if(extPeriod >0) id.period=extPeriod; 
if(id.period < LCD_DATA_PERIOD_MIN) id.period=LCD_DATA_PERIOD_MIN;                      }
//---------------------------------------------------------------------------------------
//                   КОНТРОЛЬ ИЗМЕНЕНИЯ ДАННЫХ
//--------------------------------------------------------------------------------------- 
//bool 	RT_HW_LCD_HD44780:: changeData  (RT_HW_LCD_HD44780_DataID &id, float   val) {if(id.val      !=val){id.val=val;    	return 1;} else return 0;};
bool 	RT_HW_LCD_HD44780:: changeFloat (RT_HW_LCD_HD44780_DataID &id, float   val) {if(id.valFloat !=val){id.valFloat=val; return 1;} else return 0;};
bool 	RT_HW_LCD_HD44780:: changeChar  (RT_HW_LCD_HD44780_DataID &id, char   val)  {if(id.valChar  !=val){id.valChar =val; return 1;} else return 0;};
bool 	RT_HW_LCD_HD44780:: changeInt16 (RT_HW_LCD_HD44780_DataID &id, int16_t val) {if(id.valInt16	!=val){id.valInt16=val;	return 1;} else return 0;};
bool 	RT_HW_LCD_HD44780:: changeString(RT_HW_LCD_HD44780_DataID &id, String  val) {if(id.s		!=val){id.s=val;		return 1;} else return 0;};
//---------------------------------------------------------------------------------------
//                   ПОЛУЧЕНИЕ СТРОКИ ИЗ INT16 В ФОРМАТАХ DEC,HEX,BIN
//--------------------------------------------------------------------------------------- 
String  RT_HW_LCD_HD44780:: intToString(int16_t val,char mode){
		switch(mode){
			default:  
			case 'N': return String(val);
			case 'D': return String(val,DEC);
			case 'H': return String(val,HEX);
			case 'B': return String(val,BIN);  
}}

//--------------------------------------------------------------------------------------		
RT_HW_LCD_HD44780 RT_HW_Lcd_hd44780;	//--Создание внешнего объекта		  	