	/*** FLProg ************** Библиотека RT_HW_LCD_HD44780 ver2.1 для Arduino ***************
  Вывод на несколько LCD типа HD44780 переменных String, char, int16_t, float 
  в указанное место указанного дисплея.
  Если задана длина (len) выводимого сообщения, то вывод осуществляется 
      в пределах указанной длины или до конца  строки.
  Если len=0, то вывод осуществляется до конца строк. 
  Сообщение может быть отформатировано(alig) в пределах заданной длины  влево ('L'), по центру ('C'), право ('R').
    При этом к сообщению добавляются пробелы или оно обрезается. Если alig='N' (или другой отличный символ 
	кроме 'L','C','R'), то форматирование не осуществляется. При этом надо иметь ввиду что более короткое сообщение
	не затрет предыдущее более длинное сообщение. Поэтому рекомендуется явно задавать вид форматирования или использовать
    форматирование по умолчанию, которое в библиотеке ='L' (влево).
  Имеется функция вывода мерцающего символа по указанномц адресу.	
	
  Библиотеки  разрабатываются с возможностью самостоятельного использования 
  и с учетом использования в системе визуального программирования FLProg.
  ecoins@mail.ru
  07.04.2019г.
**************************************************************************************************/
//#################################################################################################
#ifndef RT_HW_LCD_HD44780_h
#define RT_HW_LCD_HD44780_h

//#define RT_HW_LCD_HD44780_MAX_COLS  40
#define RT_HW_LCD_HD44780_VERSION 24

#include <Wire.h>

#include <stdio.h>  		//-- Библиотека структур C++
#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#define RT_HW_LCD_HD44780_DEVICE_MAX 			  8		//--Максимальное кол-во Lcd
#define RT_HW_LCD_HD44780_MAX_COLS  			 20		//--Максимальное кол-во колонок
#define RT_HW_LCD_HD44780_MAX_ROWS  			  4		//--Максимальное кол-во строк

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС ДЛЯ ХРАНЕНИЯ ПАРАМЕТРОВ LCD
// init   - устройство инциализировано
// run    - флаг разрешения запуска begin()
// link   - наличие/отсутствие( 1/0) связи с Lcd (для i2c)
// busy   - Lcd занято/свободно    (1/0) Lcd
// out    - Lcd выводит данные/команды(1) в Lcd  
//                
// num    - номер Lcd
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class  RT_HW_LCD_HD44780_DeviceID {
private:		  
struct STEP   {uint32_t initBus4:4, begin:4, send:4, send4:4, write:4, device:4, createChar:4, indexChar:4;}; 
// ****** Данные **** 4+1+1+2+8+1+1+2+4+4+40=68   
public: 
STEP		Step;   //-4 байта
char		interface;					//тип интерфейса ('I','S','P')
uint32_t 	num:4, 
            init:1, run:1, 
			link:1,  busy:1, 
			out:1,  device:1,// ok:1, 
			//sendOk:1, 
			rt:1, cntDevice:3, 
			grant:1;
uint16_t    cols:6,rows:3,errOk:1,errCode:3,errCnt:3;//сols(max=63) -кол-во столбцов Lcd, кол-во строк(max=15) в Lcd; 
uint8_t     addr;						//addr-номер пина RS при прямом подключении Lcd, или адрес i2c, или пин CS SPI;//
uint8_t		Rw,Rs,Es,DB4,DB5,DB6,DB7; 	//номера пинов при подключение Lcd через выводы Rs,Rw,E,D4,D5,D6,D7
uint8_t		delayBusy;					//задержка для ограничения загрузки линии связи
uint32_t    timeStartDelay;             //буфера для для функций getPastTense(), periodMillis() 
uint8_t     vSend;						//рабочий регистр для функции send()
uint32_t	timePeriodHot, periodHot;
uint32_t    timeNoDevice,  periodNoDevice; //буфера для для функций getPastTense(), periodMillis()
uint8_t     s_timeNoDevice;  //буфера для функции periodMillis()
char 		buff[RT_HW_LCD_HD44780_MAX_COLS];	//--буфер для вывода данных
uint8_t     buffSend[12]; 
uint16_t	divider:7, dividerWork:7; 

// ****** Методы ******
RT_HW_LCD_HD44780_DeviceID(); //--Конструктор
};
#pragma pack(pop)				//---Восстановление предыдущей настройки
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    КЛАСС ДЛЯ ХРАНЕНИЯ ОДНОГО ВЫВОДИМОГО ПАРАМЕТРА
// fresh- флаг запуска вывода сообщения
// step - 
// alig - выравнивание текста: N-без выравнивания, L-влево, C-по центру, R-вправо
// sign - выводимый символ
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_LCD_HD44780_DataID  {
public:
// ****** Данные ****    
uint32_t	lcd:4, row:4, col:6, len:6, point:2, init:1, run:1, blink:1, ok:1, blockRepeat:1, step:4;
uint32_t 	lenMax:6, lenStr:6, lenOut:6, lenBlank:6, index:6;  
char 		alig; 
char		modeInt16;					//формат вывода целого числа ('D','H','B','N')
uint32_t 	timePeriod, period;         //--буфер для периодического запуска
float 		val;						//--рабочая в некоторых функциях
String 		s;
char 		valChar;
int16_t 	valInt16; 
float		valFloat;

	//String	StringVal;

// ****** Методы ******	
RT_HW_LCD_HD44780_DataID();		//--Конструктор
	}; 
#pragma pack(pop)				//---Восстановление предыдущей настройки		
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ LCD
// Объект создается бибилиотекой и одновременнно может управлять несколькими дисплеями.
// Внутренние данные:
//    initDevice - 16-разрядный регистр флагов инициализации устройства. 
//                 Номер бита соответствует состоянию устроства с этим номером. 
//                 Флаг=1, устройство инициализировано. 
//+++++++++++++++++++++++++++++++++++++++++++++++++++_h+++++++++++++++++++++++++++++++++++
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_LCD_HD44780{
public:
// ****** ДАННЫЕ ******
RT_HW_LCD_HD44780_DeviceID *prt[RT_HW_LCD_HD44780_DEVICE_MAX];	//--Массив указателей на ID устройств
uint16_t initDevice;											//--Регистр флагов инициализации устройств	
// ****** МЕТОДЫ *****
RT_HW_LCD_HD44780(); //--Конструктор 
//--Пользовательскин функции
bool	setDeviceID (uint8_t num, RT_HW_LCD_HD44780_DeviceID &z, char interface='I',  uint8_t cols=20,uint8_t rows=4, uint8_t addr=63); //--Функция инициализация Lcd   
void 	setPeriodHot(RT_HW_LCD_HD44780_DeviceID &z, int32_t extPeriod);
bool    BEGIN       (RT_HW_LCD_HD44780_DeviceID &z);//--Инициализация и горячий перезапуск дисплея
bool    begin       (RT_HW_LCD_HD44780_DeviceID &z);//--Инициализация и горячий перезапуск дисплея с учетом флага реального времени
bool	initBus4bit (RT_HW_LCD_HD44780_DeviceID &z);//--Инициализация дисплея для обмена в 4-битовом режиме.
bool    send4       (RT_HW_LCD_HD44780_DeviceID &z, uint8_t val, uint32_t time); 				//--Отправка тетрады
bool    send        (RT_HW_LCD_HD44780_DeviceID &z, uint8_t val, uint8_t  mode, uint32_t time);	//--Отправка байта с задержкой по времени (ms)
bool    writeBuff   (RT_HW_LCD_HD44780_DeviceID &z, uint8_t len);

bool    setDataID	(uint8_t num, RT_HW_LCD_HD44780_DataID  &x, uint8_t col=0,  uint8_t row=0,  uint8_t len=0, char alig='L', uint8_t point=0); //--Функция инициализаци
void 	setPeriod   (RT_HW_LCD_HD44780_DataID   &x, int32_t extPeriod);
bool    PRINT       (RT_HW_LCD_HD44780_DataID   &x);//--Вывод строки
bool    print       (RT_HW_LCD_HD44780_DataID   &x);//--Вывод строки с учетом флага реального времени
//bool 	changeData  (RT_HW_LCD_HD44780_DataID   &x, float val); 	//--Контроль изменения параметра
bool 	changeInt16 (RT_HW_LCD_HD44780_DataID   &x, int16_t val); 	//--Контроль изменения параметра
bool 	changeChar  (RT_HW_LCD_HD44780_DataID   &x, char   val); 	//--Контроль изменения параметра
bool 	changeFloat	(RT_HW_LCD_HD44780_DataID   &x, float  val); 	//--Контроль изменения параметра
bool 	changeString(RT_HW_LCD_HD44780_DataID   &x, String val); 	//--Контроль изменения параметра
//----------------------------------------------------------------
void    setBeginWire(uint8_t num, RT_HW_LCD_HD44780_DeviceID &z,uint8_t cols,uint8_t rows,int16_t period, char RT, uint8_t divider, int16_t addr){if(num<1) num=1;
		setDeviceID((num-1),z,'I',cols,rows,addr);   setPeriodHot(z,period*1000UL); z.divider=divider;  if(RT=='Y') z.rt=1; }; //  !!! RT
void    printBegin   (RT_HW_LCD_HD44780_DeviceID &z){if(eventMillis (z.timePeriodHot, z.periodHot)) {z.run=true;} begin(z);} 
//---------------------------------------------------------------
void setBlink	(uint8_t lcd, RT_HW_LCD_HD44780_DataID &id, uint8_t col, uint8_t row, int16_t period, String val){if(lcd<1) lcd=1; if(col<1) col=1; if(row<1) row=1;
		setDataID((lcd-1),id,(col-1),(row-1));  					setPeriod(id,period);  id.blink=true; id.s=val;};            
void printBlink (RT_HW_LCD_HD44780_DataID &id) 				{if(eventMillis (id.timePeriod,id.period)){id.run=true;} print(id);};
//---------------------------------------------------------------	   
void setFloat	(uint8_t lcd, RT_HW_LCD_HD44780_DataID &id, uint8_t col, uint8_t row, int16_t period, uint8_t len, char alig, uint8_t point){ if(lcd<1) lcd=1; if(col<1) col=1; if(row<1) row=1;                           
        setDataID((lcd-1), id, (col-1), (row-1), len, alig, point);	setPeriod(id,period);  id.s=String(NAN);};                       
void printFloat (RT_HW_LCD_HD44780_DataID  &id, float val)	{if(eventMillis (id.timePeriod, id.period)){if(changeFloat(id,val)) {id.run=true; id.s=String(id.valFloat, id.point);}} print(id);}; 
//---------------------------------------------------------------	   
void setInt16	(uint8_t lcd, RT_HW_LCD_HD44780_DataID &id, uint8_t col, uint8_t row, int16_t period, uint8_t len, char alig, char mode='N'){if(lcd<1) lcd=1; if(col<1) col=1; if(row<1) row=1;                  
        setDataID((lcd-1), id, (col-1), (row-1), len, alig);		setPeriod(id,period);  id.modeInt16=mode; id.s=String(0);};                      
void printInt16 (RT_HW_LCD_HD44780_DataID  &id, int16_t val){if(eventMillis (id.timePeriod, id.period)){if(changeFloat(id,val)) {id.run=true; id.s=intToString(val,id.modeInt16);}} print(id);}; 
//---------------------------------------------------------------	   
void setChar	(uint8_t lcd, RT_HW_LCD_HD44780_DataID &id, uint8_t col, uint8_t row, int16_t period){if(lcd<1) lcd=1; if(col<1) col=1; if(row<1) row=1;                 
        setDataID((lcd-1), id, (col-1), (row-1), 1);				setPeriod(id,period);  id.s=String('*');};                      
void printChar (RT_HW_LCD_HD44780_DataID  &id, char val)	{if(eventMillis (id.timePeriod, id.period)){if(changeChar(id,val)) {id.run=true; id.s=String(val);}} print(id);}; 		 
//---------------------------------------------------------------
void setTxt		(uint8_t lcd, RT_HW_LCD_HD44780_DataID &id, uint8_t col, uint8_t row, int16_t period, uint8_t len, char alig, String txt="*", char block='N') {if(lcd<1) lcd=1; if(col<1) col=1; if(row<1) row=1;
		setDataID((lcd-1), id, (col-1), (row-1), len, alig); setPeriod(id,period);  id.s=txt; if(block=='Y') id.blockRepeat=true; else id.blockRepeat=false;}; 	
void printTxt	(RT_HW_LCD_HD44780_DataID  &id){if(!id.blockRepeat) {if(eventMillis (id.timePeriod,id.period)){id.run=true;}} print(id);};		 
//---------------------------------------------------------------	
void setString(uint8_t lcd, RT_HW_LCD_HD44780_DataID &id, uint8_t col, uint8_t row, int16_t period, uint8_t len, char alig, char block='N') {if(lcd<1) lcd=1; if(col<1) col=1; if(row<1) row=1;
		setDataID((lcd-1), id, (col-1), (row-1), len, alig); setPeriod(id,period);  id.s="*"; if(block=='Y') id.blockRepeat=true; else id.blockRepeat=false;}; 	
void printStr(RT_HW_LCD_HD44780_DataID  &id, String val){if(eventMillis (id.timePeriod,id.period)){if(id.s!=val) {id.s!=val; id.run=true; }} print(id);};		 
//---------------------------------------------------------------	 
bool   dividerTask(RT_HW_LCD_HD44780_DeviceID  &id) {
	if(id.dividerWork==0) id.grant=true; else id.grant=false;
	id.dividerWork++; 
	if((id.dividerWork>=id.divider) || (id.dividerWork>=100)) id.dividerWork=0;
	return id.grant;};	
//---------------------------------------------------------------
bool	getGrant (uint8_t num) {return 1;};	
//---------------------------------------------------------------
String  intToString (int16_t val,char mode);		//--Преобразование целого в строку в форматах DEC,HEX,BIN 	
uint8_t pointCursor (uint8_t col, uint8_t row);		//--Вычисление позиции курсора в памяти Lcd	
//---------------------------------------------------------------
uint32_t getPastTense	 (uint32_t &t1, uint32_t t2);		//--ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ 
bool 	 eventMillis	 (uint32_t &t1, uint32_t val);	    //--ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
bool     periodMillis    (uint8_t  &s, uint32_t &t1, uint32_t v);	//--событие с периодичностью val(ms)
//---------------------------------------------------------------
bool 	createChar    (uint8_t location, uint8_t charmap[], uint8_t lcd=0);//--Загрузка до 8 символов в CGRAM  ?? В работе
bool    loadChar      (uint8_t char_num, uint8_t *rows,     uint8_t lcd=0);//  ?? В работе   
};  
#pragma pack(pop)				//---Восстановление предыдущей настройки
//==============  Конец класса ======================================================
extern RT_HW_LCD_HD44780 RT_HW_Lcd_hd44780; //--Создание объекта LCD типа HD44780
#endif
//===========================================================================================



