#include "RT_HW_HC_SR04.h"
//-------------------------------------------------------------------------------------------------
//          Параметры
//-------------------------------------------------------------------------------------------------
#define SR04_timePulse	     10	//--Длительность (mcs) генерируемого импульса на пине Trig 
#define SR04_maxBegin       700	//--Максимальное время (mcs) появления ответного импульса Echo 
#define SR04_minBegin       200	//--Минимальное время  (mcs) появления ответного импульса Echo 
#define SR04_maxDurate    25000	//--Максимальная длительность ответного импульса
#define SR04_maxCmYesIRQ    150	//--Уставка максимального расстояния(см), если пин Echo с прерываниями
#define SR04_maxCmNoIRQ      70	//--Уставка максимального расстояния(см), если пин Echo без прерывания
#define SR04_periodYesIRQ    20 //--Периодичность запуска с пином c прерыванием
#define SR04_periodNoIRQ    200 //--Периодичность запуска с пином без прерывания
#define SR04_averYesIRQ       3 //--Усреднение измерений с пином c прерыванием
#define SR04_averNoIRQ        2 //--Усреднение измерений с пином без прерывания
#define SR04_maxErrNoInit     8 //
#define SR04_maxErrTimeFrontMin  5 //
#define SR04_maxErrTimeFrontMax  5 //
#define SR04_maxErrTimeEnd    5 //
#define SR04_maxErrNoFront    5 //
#define SR04_maxErrTimeOut    8 //
#define SR04_maxMeterNull     3 //
#define SR04_maxMeterMax      3 //
#define SR04_convert        582 //--Коэффициент пересчета мкс в мм
#define SR04_period_Min      20 //--Минимальный период вызова  
#define SR04_modeOut          3 //--Код выводимых данных 
#define SR04_coeffCmToMcs	 70 //--Коэффициент преобразования 
//-------------------------------------------------------------------------------------------------
//          RT_HW_HC_SR04_ID:: КОНСТРУКТОР
//-------------------------------------------------------------------------------------------------
RT_HW_HC_SR04_ID:: RT_HW_HC_SR04_ID(){init=false;};
//-------------------------------------------------------------------------------------------------
//          RT_HW_HC_SR04_ID:: ОБРАБОТЧИК ПРЕРЫВАНИЙ
//-------------------------------------------------------------------------------------------------
void RT_HW_HC_SR04_ID:: IRQ_handler(){
if(!start) return;
if(!begin)  { if( digitalRead(pinEcho)) {timeBegin=micros();  begin=true;                           }}
  else      { if(!digitalRead(pinEcho)) {timeEnd  =micros();  end  =true;  detachInterrupt(numIRQ); }}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
//           Инициализация устройств (возвращает id.init=; 	
//-------------------------------------------------------------------------------------------------
bool RT_HW_HC_SR04:: setID(RT_HW_HC_SR04_ID &id, int16_t Echo, int16_t Trig) {uint8_t n;
	if(id.init) 				return 1;      //--Выход без изменения параметров инцициализации, если устройство уже инициализировано
	if(Echo<0) return id.init;	//-Выход, если пин Echo не определен		  
	id.pinEcho=Echo; if(Trig<0) id.pinTrig=id.pinEcho; else id.pinTrig=Trig; //--Установка номера пинов 
	id.numIRQ=digitalPinToInterrupt(id.pinEcho);			//  Если пин не имеет прерывания, возвращается -1
	if(id.numIRQ <0){id.modeIRQ=0; id.LenghtMaxCm =SR04_maxCmNoIRQ;  id.period=SR04_periodNoIRQ;  id.aver=SR04_averNoIRQ;}//--Параметры для пина без прерывания 
	            else{id.modeIRQ=1; id.LenghtMaxCm =SR04_maxCmYesIRQ; id.period=SR04_periodYesIRQ; id.aver=SR04_averYesIRQ;}//--Параметры для пина  с прерыванием 	
	id.stepIRQ=0; 	    		//--Очистка параметров
	id.runIRQ=false;
	id.timePeriod=millis();	//--Установка времени
	id.modeOut=SR04_modeOut;//--Режим вывода:=0 -мкс;=1 -мм; =2 -см; 3 -мкс+усреднение; 4 -мм+усред.; 5 -см+усред.
	id.LenghtMcs=id.LenghtMaxCm * SR04_coeffCmToMcs;
	id.regimeIRQ=id.modeIRQ;
	id.fresh=id.meterOk=false;
	id.start =true;			//--Установка флага запуска для первого измерения
	id.init=true;			//--Установка флага инициализации
return 1;
}; //--Нормальный выход после успешной инициализации
//-------------------------------------------------------------------------------------------------
//		           ЗАПУСК ИЗМЕРЕНИЯ
//-------------------------------------------------------------------------------------------------
bool RT_HW_HC_SR04:: makeTrig(RT_HW_HC_SR04_ID &id) {
    id.start = id.begin = id.end = false;	//--Сброс флагов состояни ответного импульса
  //------ЗАПУСК ЗВУКОВОГО ИМПУЛЬСА----------------------------------
	digitalWrite(id.pinTrig, LOW);			//--Пин в низкий уровень, чтобы не было помех при настраивании на выход         
	pinMode		(id.pinTrig, OUTPUT );		//--Настраиваем как выход	
	digitalWrite(id.pinTrig, HIGH);   		//--Устанавливаем высокий уровень (начало генерации запускающего импульса)      
	delayMicroseconds(SR04_timePulse);		//--Задержка на длительность генерируемого импульса на пине Trig 
	digitalWrite(id.pinTrig, LOW);    		//--Завершаем запускающий импульс 
	pinMode		(id.pinEcho, INPUT_PULLUP);	//--Настраиваем пин Echo как вход
 //------------------------------------------------------------------
	if(digitalRead(id.pinEcho)) return 0;	//ПРОВЕРКА ВХОДА pinEcho.Выход по ошибке, если на входе высокий уровень
 //-------УСТАНОВКА НАЧАЛЬНОГО ВРЕМЕНИ-------------------------------	
	id.timeStart=id.timeBegin=id.timeEnd =micros(); //--Установка начальных времен для измерения параметров ответного импульса
    id.start=true; id.begin=id.end= false;	//--Установка флагов состояния ответного импульса
return 1;}; //--Выход    
//-------------------------------------------------------------------------------------------------
//		   ФИКСАЦИЯ СОБЫТИЙ С ПИНОМ БЕЗ АППАРАТНОГО ПРЕРЫВАНИЯ
//-------------------------------------------------------------------------------------------------
bool RT_HW_HC_SR04:: meterNoIRQ(RT_HW_HC_SR04_ID &id) {uint32_t maxVal;
	if((!id.init) || (!id.run)) return 0; id.run=false; //--Выход если устройство не инициализировано или не взведен флаг запуска измерения
	makeTrig(id);						  				//--Генерация запускающего импульса (вход Trig)
	maxVal=id.LenghtMaxCm * SR04_coeffCmToMcs;			  				//--Вычисление максимальной дистанции в mcs
  //------Обнаружение переднего фронта ответного импульса Echo                        
     while(1){ if(digitalRead(id.pinEcho))    {id.timeBegin=micros();                 id.begin=true; break;} // Фиксация времени переднего фронта и установка флага прихода переднего фронта 
			   if(RT_HW_hc_sr04.getPastTense(id.timeStart,micros()) > SR04_maxBegin) {id.start=false; break;}}	//--Сброс флага запуска по тайм-ауту
  //------Обнаружение заднего фронта ответного импульса Echo   			
	if(id.begin)
	{while(1){ if(!digitalRead(id.pinEcho))	  {id.timeEnd=  micros();    id.end=true; id.start=false; break;}	
	           if(RT_HW_hc_sr04.getPastTense(id.timeStart,micros()) > maxVal) 
                   {id.timeEnd=id.timeBegin+maxVal;                      id.end=true; id.start=false; break;}}}
  //------Обработка результатов измерения	
	meterOutLook(id);
return 1;}; //--Выход
//-------------------------------------------------------------------------------------------------
//		   ФИКСАЦИЯ СОБЫТИЙ С ПИНОМ C АППАРАТНЫМ ПРЕРЫВАНИЕМ
//-------------------------------------------------------------------------------------------------
bool RT_HW_HC_SR04:: meterYesIRQ(RT_HW_HC_SR04_ID &id) {
	switch(id.stepIRQ){default: id.stepIRQ=0;
		case 0: if((!id.init) || (!id.run)) return 0; 
		id.run=false; id.stepIRQ++;//--Выход если устройство не инициализировано или не взведен флаг запуска измерения
		case 1: detachInterrupt(id.numIRQ); 
		        makeTrig(id);
				id.runIRQ=1; 						id.stepIRQ++;
		case 2: if(id.runIRQ) 			return 0;	id.stepIRQ++;
		case 3: if(! meterOutLook(id)) 	return 0; 	id.stepIRQ=0; 
		return 1;}};
//-------------------------------------------------------------------------------------------------
//		   ИЗМЕРЕНИЕ C АППАРАТНЫМ ПРЕРЫВАНИЕМ ИЛИ БЕЗ НЕГО
//-------------------------------------------------------------------------------------------------
bool RT_HW_HC_SR04:: meter(RT_HW_HC_SR04_ID &id){if(id.regimeIRQ){return RT_HW_hc_sr04.meterYesIRQ(id);} else {return RT_HW_hc_sr04.meterNoIRQ(id);}}		
		
//-------------------------------------------------------------------------------------------------
//		   ОБРАБОТКА РЕЗУЛЬТАТОВ ИЗМЕРЕНИЯ    !!!
//-------------------------------------------------------------------------------------------------	  
bool RT_HW_HC_SR04:: meterOutLook(RT_HW_HC_SR04_ID &id) {uint32_t g; uint32_t maxVal; uint16_t meterCur; uint16_t k;
maxVal=id.LenghtMaxCm*SR04_coeffCmToMcs; //--Максимальное значение с запасом	
//--Обработка параметров ответного импульса
g=RT_HW_hc_sr04.getPastTense(id.timeStart,id.timeBegin);
	if(id.begin){  if(g<=SR04_minBegin) {id.codeErr=2;  goto pointEndMeterLook;}  //--Передний импульс появился после стартового за пределами диапазона меньше 200 или больше 700 mcs  
				   if(g >SR04_maxBegin) {id.codeErr=3;  goto pointEndMeterLook;}  //--Передний импульс появился после стартового за пределами диапазона меньше 200 или больше 700 mcs  		
	     if(id.end) {					 id.codeErr=0;  goto pointEndMeterLook;}  //--Возвратился нормальный  ответный импульс, запоминаем результат
         else     {if(g >maxVal)	    {id.codeErr=4;  goto pointEndMeterLook;}}}//--Превышено время ожидания прихода окончания обратного импульса.
	else{      	   if(g >SR04_maxBegin)	{id.codeErr=5;  goto pointEndMeterLook;}} //--Нет переднего фронта ответного импульса (нет датчика)	      
//--Обработка защиты от зависания
g=RT_HW_hc_sr04.getPastTense(id.timeStart,micros());	
				   if(g >SR04_maxDurate){id.codeErr=6;	goto pointEndMeterLook;}  //--Защита от зависания 
//--Выход для продолжения измерения	в следующем цикле					   
return 0; 																		  																							 
//-------ОБРАБОТКА ОШИБОК
pointEndMeterLook:   														//--Метка для операторов goto в данной функции	  
 digitalWrite(id.pinTrig,LOW); pinMode(id.pinTrig, OUTPUT); id.start=false; //--Останавливаем цикл измерений
// maxVal=id.LenghtMaxCm*70; //--Максимальное значение с запасом
     //--Обработка результата
	 switch(id.codeErr)
	 { default: id.codeErr=7; break;
		case 0: id.LenghtMeter=RT_HW_hc_sr04.getPastTense(id.timeBegin,id.timeEnd); //if(id.LenghtMeter >maxVal) id.LenghtMeter=maxVal; 	//--Фиксируем успешное измерение
				//--Сброс ошибок и флагов
				id.cntErrNoInit =id.cntErrTimeFrontMin =id.cntErrTimeFrontMin =id.cntErrNoFront =id.cntErrTimeEnd =id.cntErrTimeOut=0;	//--Сбрасываем ошибки
				id.noSensor=id.errSensor=false; 				//--Сбрасываем флаги
				//---
				
				break; //--
		case 1: if (++id.cntErrNoInit       >SR04_maxErrNoInit) 	  {id.cntErrNoInit=0; 	  	id.LenghtMeter=maxVal;  id.errSensor=true;} break; //--Фронт ответного импульса не в пределах рабочего диапозогна. При повторении, фиксируем результат=0 и ошибку.
		case 2: if (++id.cntErrTimeFrontMin >SR04_maxErrTimeFrontMin) {id.cntErrTimeFrontMin=0; id.LenghtMeter=maxVal;	id.errSensor=true;} break; //--Фронт ответного импульса не в пределах рабочего диапозогна. При повторении, фиксируем результат=0 и ошибку.
		case 3: if (++id.cntErrTimeFrontMax >SR04_maxErrTimeFrontMax) {id.cntErrTimeFrontMax=0;	id.LenghtMeter=maxVal;	id.errSensor=true;} break; //--Фронт ответного импульса не в пределах рабочего диапозогна. При повторении, фиксируем результат=0 и ошибку.
		case 4: if (++id.cntErrTimeEnd      >SR04_maxErrTimeEnd) 	  {id.cntErrTimeEnd=0;      id.LenghtMeter=maxVal;                    } break; //--Превышение ожидания заднего фронта ответного импульса. При повторение, фиксируем результат=max (без ошибки).
		case 5: if (++id.cntErrNoFront      >SR04_maxErrNoFront) 	  {id.cntErrNoFront=0;      id.LenghtMeter=maxVal;	id.errSensor=true;} break; //--Нет переднего фронта ответного импульса (нет датчика). При повторяющейся ошибки фисируем результат=0 и флаг отсутствия сенсора=1; 	
		case 6: if (++id.cntErrTimeOut      >SR04_maxErrTimeOut) 	  {id.cntErrTimeOut=0;		id.LenghtMeter=2*SR04_coeffCmToMcs;	id.noSensor =true; id.errSensor=true;} break; //--При повторяющейся ошибки фисируем результат=0, флаг ошибки. 		
 }	
//------------------------- 
id.meterOk=true;							//--Устанавливаем флаг завершения измерения
meterCur=id.LenghtMeter;
if(meterCur==0) 	{if(++id.cntMeterNull >SR04_maxMeterNull) {id.cntMeterNull=0; meterCur=3*SR04_coeffCmToMcs;}};
if(meterCur>maxVal) {if(++id.cntMeterMax  >SR04_maxMeterMax)  {id.cntMeterMax=0;  meterCur=maxVal;}}; 
if ((meterCur>0) && (meterCur<=maxVal)) {id.cntMeterNull=id.cntMeterMax=0;};
if(id.LenghtMcs!=meterCur) id.fresh=true; 	//--Устанавливается флаг свежих данных
id.LenghtMcs=meterCur;						//--Сохраняем вычисленное значение
 //------ПОЛУЧЕНИЕ УСРЕДНЕННОГО ЗНАЧЕНИЯ в mcs 
	maxVal=id.LenghtAver;
	if(id.aver >0)	{
		maxVal *=id.aver;		//--Предыдущий результат умножается на коэффициент усреднения
		maxVal +=id.LenghtMcs;  //--Полученное значение увеличивается на текущий результат
		maxVal /=(id.aver+1);}	//--Полученное значение делится на коэффициент усреднения, увеличенный на 1			 
   else{maxVal  =id.LenghtMcs;}	//--Если усреднение результата не требуется, то запоминается текущий результат
	id.LenghtAver=maxVal;
return 1;}; 
//-------------------------------------------------------------------------------------------------
//		   КОНВЕНТИРОВАНИЕ mcs текущее в мм
//-------------------------------------------------------------------------------------------------
uint16_t  RT_HW_HC_SR04::convertMcs     (RT_HW_HC_SR04_ID &id) {uint32_t x;
x=uint32_t(id.LenghtMcs) *100/SR04_convert; if(x >(id.LenghtMaxCm*10)) x=id.LenghtMaxCm*10; return x;}
//-------------------------------------------------------------------------------------------------
//		   КОНВЕНТИРОВАНИЕ mcs усредненное в мм
//-------------------------------------------------------------------------------------------------
uint16_t  RT_HW_HC_SR04::convertAver    (RT_HW_HC_SR04_ID &id) {uint32_t x;
x=uint32_t(id.LenghtAver) *100/SR04_convert; if(x >(id.LenghtMaxCm*10)) x=id.LenghtMaxCm*10; return x;}
//-------------------------------------------------------------------------------------------------
//		   ПОЛУЧЕНИЕ ДАННЫХ 
//-------------------------------------------------------------------------------------------------
uint16_t  RT_HW_HC_SR04::outVal         (RT_HW_HC_SR04_ID &id, int16_t mode)
{ switch(mode){
     default:
	 case 0:  return convertMcs(id)/10;   //--Вывод в см 
	 case 1:  return id.LenghtMcs;    	  //--Вывод в мкс
     case 2:  return convertMcs(id);      //--Вывод в мм
     case 3:  return convertMcs(id)/10;   //--Вывод в cm
     case 4:  return id.LenghtAver;       //--Вывод в мкс усредненное
     case 5:  return convertAver(id);     //--Вывод в мм  усредненное
     case 6:  return convertAver(id)/10;  //--Вывод в см  усредненное (по умолчанию) 
}}; 
//---------------------------------------------------------------------------------------
//         ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ
//---------------------------------------------------------------------------------------
uint32_t RT_HW_HC_SR04:: getPastTense (uint32_t &t1, uint32_t t2){if (t2>=t1) {return (t2-t1);} else {return (0xFFFFFFFF-t1+t2);}};  
//---------------------------------------------------------------------------------------
//            ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
//---------------------------------------------------------------------------------------
 bool    RT_HW_HC_SR04:: eventMillis (uint32_t &t1,uint32_t val){if(getPastTense (t1,millis())>val) {t1+=val; return 1;} return 0;};
//---------------------------------------------------------------------------------------
//                   УСТАНОВКА ПЕРИОДА ОБНОВЛЕНИЯ ДАННЫХ  
//--------------------------------------------------------------------------------------- 
void 	RT_HW_HC_SR04:: setPeriod(RT_HW_HC_SR04_ID &id, int32_t extPeriod) { 
if(extPeriod >0) id.period=extPeriod; 
if(id.period < SR04_period_Min) id.period=SR04_period_Min; }    
//---------------------------------------------------------------------------------------		
RT_HW_HC_SR04 RT_HW_hc_sr04;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++






