/******************************************************************
  Библиотека для семейства ультразвуковых дальномеров  HC-SR04
	в систем реального времени типа 
Версия 1.1 
******************************************************************/
#ifndef RT_HW_HC_SR04_h
#define RT_HW_HC_SR04_h

#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
#define RT_HW_HC_SR04_VER 12
//-------------------------------------------------------------------------------------------------
//			             КЛАСС ДАННЫХ HC-SR04
// 					(размещение кратное 4-байт - всего 36 байт на сонар)
// aver - коэффициент усреднения показаний(<=0-без усреднения, 1-минимальное усреднение, ... 63-максимальное усреднение)
//-------------------------------------------------------------------------------------------------
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_HC_SR04_ID {
public:	//2+4+2+6+2+20=36 байт
	int16_t		pinEcho, pinTrig, numIRQ;
	uint32_t	init:1, initAdd:1, modeIRQ:1, regimeIRQ:1, noSensor:1,
				run:1,  start:1,   begin:1,  end:1,  errSensor:1, meterOk:1, fresh:1, 
				//big:1, 
				aver:6,  runIRQ:1, stepIRQ:4, modeOut:4;	
	uint32_t 	codeErr:4, 
	             cntErrNoInit:4,		//--codeErr=1
				 cntErrTimeFrontMin:4,	//--codeErr=2
				 cntErrTimeFrontMax:4,	//--codeErr=3//-- 
				 //cntErrTimeFront:4, 	//--codeErr=
				 cntErrNoFront:4,		//--codeErr=4 
				 cntErrTimeEnd:4, 		//--codeErr=5
				 cntErrTimeOut:4;		//--codeErr=6
	uint8_t		cntMeterNull:4, cntMeterMax:4;
	uint16_t 	LenghtMeter, LenghtMaxCm, LenghtMcs, LenghtAver; 
	uint32_t    period;
	uint32_t	timeStart, timeBegin, timeEnd, timeMeter, timePeriod, timeFLProg;
RT_HW_HC_SR04_ID(); //--Конструктор
void IRQ_handler(); //--Обработчик прерываний
};
#pragma pack(pop)				//---Восстановление предыдущей настройки
//-------------------------------------------------------------------------------------------------
//			             КЛАСС ДЛЯ СОНАРА HC-SR04
// 					(размещение кратное 4-байт - всего 36 байт на сонар)
// averaging - коэффициент усреднения показаний(0-без усреднений, 1-минимальное усреднение, ... 100-сильное усреднение, 255-max усреднение)
//-------------------------------------------------------------------------------------------------
#pragma pack(push, 4)		//--Установка выравнивания до 4 байт
class RT_HW_HC_SR04{
public:
// ****** Методы ******
RT_HW_HC_SR04(){}; //--Конструктор	
//bool	 setDevice	 (RT_HW_HC_SR04_ID &id, int16_t Echo=-1, int16_t Trig=-1); //--Инициализация устройства
bool     setID	 	 (RT_HW_HC_SR04_ID &id, int16_t Echo=-1, int16_t Trig=-1); //--Инициализация данных устройтсва
bool	 makeTrig	 (RT_HW_HC_SR04_ID &id);				  //--Запуск измерения
bool   	 meter       (RT_HW_HC_SR04_ID &id);               //--Измерение: если =1 ->meterYesIRQ(id); иначе ->meterNoIRQ(id)
bool     meterNoIRQ	 (RT_HW_HC_SR04_ID &id);				  //--Измерение без прерывания
bool	 meterYesIRQ (RT_HW_HC_SR04_ID &id);				  //--Измерение с прерываниями
bool     meterOutLook(RT_HW_HC_SR04_ID &id);				  //--Обработка измеренных данных
uint16_t convertMcs	 (RT_HW_HC_SR04_ID &id);				  //--Получение результата mcs
uint16_t convertAver (RT_HW_HC_SR04_ID &id);				  //--Получение результата mcs с усреднением
uint16_t outVal		 (RT_HW_HC_SR04_ID &id, int16_t mode); //--Возвращает измеренные данные по типу данных
void     setPeriod   (RT_HW_HC_SR04_ID &id, int32_t external=-1); //--ЗАПУСК НА ИСПОЛНЕНИЕ В FLPROG 		
//--------------------------------------------------------------------------------------------------
uint32_t getPastTense(uint32_t &t1, uint32_t t2);		//--ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ 
bool 	 eventMillis (uint32_t &t1, uint32_t val);	    //--ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
};
#pragma pack(pop)				//---Восстановление предыдущей настройки
//-------------------------------------------------------------------------------------------------
//==============  Конец класса ======================================================
extern RT_HW_HC_SR04 RT_HW_hc_sr04; //--Создание объекта LCD типа HD44780
//===================================================================================
#endif
