#include "RT_HW_DHTx.h"
//-------------------------------------------------------------------------------------------------
//          Параметры
//-------------------------------------------------------------------------------------------------
#define DHTx_period_Min      	2200 //--Минимальный период вызова в мс
#define DHTx_period_Lib			2500 //--Минимальный период вызова в мс
#define DHT22_DELAY_BEFORE_METER 600 //--ЗАДЕРЖКА ПЕРЕД ЗАПУСКОМ ИЗМЕРЕНИЯ ДЛЯ DHT22
#define DHT22_CNT_ERR_MAX    	   3 //--Максимальное кол-во шибок
//-------------------------------------------------------------------------------------------------
//          RT_HW_DHTx_ID:: КОНСТРУКТОР
//-------------------------------------------------------------------------------------------------
RT_HW_DHTx_ID:: RT_HW_DHTx_ID(){init=initAdd=false; extRUN=true;};
//-------------------------------------------------------------------------------------------------
//          RT_HW_HC_SR04_ID:: ОБРАБОТЧИК ПРЕРЫВАНИЙ
//-------------------------------------------------------------------------------------------------
void RT_HW_DHTx_ID:: IRQ_handler(){
//if(!start) return;
//if(!begin)  { if( digitalRead(pinEcho)) {timeBegin=micros();  begin=true;}}
//  else      { if(!digitalRead(pinEcho)) {timeEnd  =micros();  end  =true;  detachInterrupt(numIRQ);}}
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
//                           ИНИЦИАЛИЗАЦИЯ
//-------------------------------------------------------------------------------------------------
bool RT_HW_DHTx:: setID(RT_HW_DHTx_ID &id, int16_t pin) {
	if(id.init)      return 1;	//--Выход, если устройство уже инициализировано 
	if(pin <0)       return 1;	//--Выход, если пин не определен
	id.pin=pin; id.init=true;
	id.numIRQ=digitalPinToInterrupt(id.pin);
	if(id.numIRQ <0){id.modeIRQ=0;} //--Параметры для пина без прерывания 
	            else{id.modeIRQ=1;} //--Параметры для пина  с прерыванием 	
	id.stepStart=id.stepMeter=id.stepConvert=0;
	id.startOk=  id.meterOk=  id.convertOk=false; 
	id.codeErr=  id.cntErr=   id.cntMeter=0; id.maxErr=DHT22_CNT_ERR_MAX;
	id.temperature=id.humidity=NAN;
	id.period=DHTx_period_Lib;
return 1;};
//-------------------------------------------------------------------------------------------------
//            ЧТЕНИЕ СЕНСОРА В РЕЖИМЕ RT
// 1.Линия сенсора притягивается к земле - менее 500mсs
// 2.Линия сенсора переводитс¤ в режим чтения.
// 3.Последовательно принимается 83 изменения линии:
//    FALLING, RISING и FALLING - это стартовый бит, затем 40 бит: 
//         RISING, а затем FALLING за бит принимается любое значение HIGH или LOW, если оно не превышает 85mcs, 
//    иначе ошибка по тайм-ауту     ---> codeErr=2.
// 4. Проверяется контрольная сумма ---> codeErre=3.
// 5. При отсутствии ошибок запоминаются полученные результаты.
// 6. После кол-ва ошибочных чтений, превышающих maxErr, результатам T и H присваивается NAN
//  коды ошибок:
//	=0 - нет ошибок
//  =1 - нет инициализации
//  =2 - ошибка по тайм-ауту
//	=3 - ошибка по контрольной сумме
//	=4 - выход за диапазон измерений
//-------------------------------------------------------------------------------------------------
bool RT_HW_DHTx:: meter(RT_HW_DHTx_ID &id){
switch(id.stepMeter){
	default: id.stepMeter=0;
	case 0: 
			if(!meterStart(id)) 									return 0;
         	if(id.codeErr==0) id.stepMeter++; else {id.stepMeter=2; return 0;}
    case 1: //if(id.modeIRQ) {}
	        //         else {if(meterNoIRQ(id)) {id.stepMeter++;}  else {return 0;}}
			if(meterNoIRQ(id)) {id.stepMeter++;} else {return 0;}
	case 2: convert(id);
			id.stepMeter=0; return 1;	
	} return 1;}; 
//-------------------------------------------------------------------------------------------------
//                           ЗАПУСК ИЗМЕРЕНИЯ
//-------------------------------------------------------------------------------------------------
bool RT_HW_DHTx:: meterStart(RT_HW_DHTx_ID &id){	
	if (!id.run)    						{return 0;} //--ВЫХОД  ЕСЛИ ИЗМЕРЕНИЕ СОНАРА НЕ ЗАПУЩЕНО 
	if (!id.init) {id.codeErr=1; id.run=0;   return 0;} //--ВЫХОД  ЕСЛИ СОНАРА НЕ ИНИЦИИРОВАН
	id.codeErr=0; id.run=0;								//--Сброс кода ошибки, сброс флага запуска, переход
	digitalWrite(id.pin, LOW); pinMode(id.pin, OUTPUT); //--Притягиваем линия сенсора к земле 
	delayMicroseconds(DHT22_DELAY_BEFORE_METER);
   id.stepStart=0; id.startOk=true; return 1;}
//-------------------------------------------------------------------------------------------------
//                     ИЗМЕРЕНИЯ НА ПИНЕ БЕЗ ПРЕРЫВАНИЙ
//-------------------------------------------------------------------------------------------------
bool RT_HW_DHTx:: meterNoIRQ(RT_HW_DHTx_ID &id){uint8_t age;  
	pinMode(id.pin, INPUT);  digitalWrite(id.pin, HIGH);//--Переводим пин в режим чтения и читаем
    //--Начала цикла изменений на входе (83 раз) - первые три - стартовый импульс
    id.data=0;	
	for (int8_t i = -3 ; i < 2 * 40; i++ ) 
	 {id.timeStartDelay=micros();
	  do {age = (uint32_t)getPastTense(id.timeStartDelay, micros()); 
	      if(age >90){id.codeErr=2; id.meterOk=true; return 1;}} //--Ожидание импульса - если ожидание превысило 90 - выход по ошибке тайм-аута =2    
	   while (digitalRead(id.pin) == (i & 1) ? HIGH : LOW );				
      if (i>= 0 && (i & 1)) {id.data <<= 1; if ( age > 30 ) {id.data |= 1;} }//--Проверяем изменение на линии (первые три стартовых импульса пропускаем)
      if (i==31) {id.rawHumidity =    id.data; }
      if (i==63) {id.rawTemperature = id.data; id.data = 0;}
     } 
	id.meterOk=true;
return 1;};
//-------------------------------------------------------------------------------------------------
//                     ИЗМЕРЕНИЯ НА ПИНЕ С ПРЕРЫВАНИЯМИ
//-------------------------------------------------------------------------------------------------
bool RT_HW_DHTx:: meterYesIRQ(RT_HW_DHTx_ID &id){uint8_t age;  
	switch(id.stepYesIRQ){default: id.stepYesIRQ=0;
	case 0: return 0;
	case 1: pinMode(id.pin, INPUT);  digitalWrite(id.pin, HIGH); break;//--Переводим пин в режим чтения и читаем
	}
	/*	
    //--Начала цикла изменений на входе (83 раз) - первые три - стартовый импульс
    id.data=0;	
	for (int8_t i = -3 ; i < 2 * 40; i++ ) 
	 {id.timeStartDelay=micros();
	  do {age = (uint32_t)getPastTense(id.timeStartDelay, micros()); 
	      if(age >90){id.codeErr=2; id.meterOk=true; return 1;}} //--Ожидание импульса - если ожидание превысило 90 - выход по ошибке тайм-аута =2    
	   while (digitalRead(id.pin) == (i & 1) ? HIGH : LOW );				
      if (i>= 0 && (i & 1)) {id.data <<= 1; if ( age > 30 ) {id.data |= 1;} }//--Проверяем изменение на линии (первые три стартовых импульса пропускаем)
      if (i==31) {id.rawHumidity =    id.data; }
      if (i==63) {id.rawTemperature = id.data; id.data = 0;}
     } 
	id.meterOk=true;
*/	
return 1;
};
//-------------------------------------------------------------------------------------------------
//                           ОБРАБОТКА ИЗМЕРЕНИЯ
//-------------------------------------------------------------------------------------------------
bool RT_HW_DHTx:: convert(RT_HW_DHTx_ID &id){ float _humidity, _temperature;  
    if(id.codeErr!=0) goto pointErr;
//---Обработка данных на достоверность и запоминаем если нет ошибок, иначе выход ошибке  — =3	
	if( (uint8_t)(((uint8_t)id.rawHumidity) + (id.rawHumidity >> 8) + ((uint8_t)id.rawTemperature) + (id.rawTemperature>>8)) !=id.data)  {id.codeErr=3;  goto pointErr;}	
//---Запоминаем результат
	_humidity = id.rawHumidity * 0.1;   
	if ( id.rawTemperature & 0x8000 ) { id.rawTemperature = -(int16_t)(id.rawTemperature & 0x7FFF);}
	_temperature = ((int16_t)id.rawTemperature) * 0.1;
//--Проверяем полученные измерения на заданный диапазон
    if( (_humidity <0) || (_humidity > 100)  || (_temperature < -50) || (_temperature > 125)) {id.codeErr=4; goto pointErr;} 
//-------------------------------	
	id.error=0; id.cntErr=0;  id.codeErr=0;
	if((id.humidity!=_humidity) || (id.temperature!=_temperature)) {id.fresh=true; id.humidity=_humidity; id.temperature=_temperature;}
	return 1;
//--Обработка ошибки
pointErr:	
	id.cntErr++; if(id.cntErr>=id.maxErr) {id.cntErr=0; id.error=true; id.fresh=true; id.temperature=id.humidity=NAN;} 
	//Serial.println("codeErr="+ String(id.codeErr)+ " cntErr="+String(id.cntErr)+ " t=" + String(id.temperature,2));		
return 1;}
//---------------------------------------------------------------------------------------
//         ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ
//---------------------------------------------------------------------------------------
uint32_t  RT_HW_DHTx:: getPastTense (uint32_t &t1, uint32_t t2){if (t2>=t1) {return (t2-t1);} else {return (0xFFFFFFFF-t1+t2);}};  
//---------------------------------------------------------------------------------------
//            ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
//---------------------------------------------------------------------------------------
 bool     RT_HW_DHTx:: eventMillis (uint32_t &t1,uint32_t val){if(getPastTense (t1,millis())>val) {t1+=val; return 1;} return 0;};
 //---------------------------------------------------------------------------------------
//  ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ (более точно и не зависимо чем event)  
//---------------------------------------------------------------------------------------
bool 	  RT_HW_DHTx:: periodMillis(uint8_t &s,uint32_t &t1,uint32_t v){if(s==0){t1=millis();s=1;}if(getPastTense(t1,millis())>v){t1=millis();return 1;}return 0;} 
//---------------------------------------------------------------------------------------
//                   УСТАНОВКА ПЕРИОДА ОБНОВЛЕНИЯ ДАННЫХ  
//--------------------------------------------------------------------------------------- 
void 	RT_HW_DHTx:: setPeriod(RT_HW_DHTx_ID &id, int32_t extPeriod) { 
if(extPeriod >0) id.period=extPeriod; 
if(id.period < DHTx_period_Min) id.period=DHTx_period_Min; }    
//---------------------------------------------------------------------------------------		
RT_HW_DHTx RT_HW_dhtX;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++