/**************************************************************************************************
 *             Тест датчика DHT-22(сенсор) в режиме RT
 *  1. В секции DECLARE (до секции setup()) создать объект: RT_DHT22 xxx. 
 *     При создании объекта инициализирутся переменные объекта: 
 *     bool     permit  =true;  //Флаг "Разрешение измерения сенсора" (=true) 
 *     bool     start   =true;  //Флаг "Измерение выполнено" (=true - сбрасывается при следующем входе в функцию run()
 *     bool     error   =false; //Флаг "Измерение завершено с ошибкой"(=true - сбрасывается при успешном чтении) 
 *     bool     initOk  =false; //Флаг "Инициализация сенсора выполнена"(=true)  

 *     uint8_t  pin     =99;    //Номер пина 
 *     float temperature= NAN;  //Температура сенсора
 *     float humidity   = NAN;  //Влажность сенсора
 *     uint8_t  maxError=3;     //Максимальное кол-во повторных ошибок*   
 *     uint8_t codeError=0;     //Код завершения последнего чтения сенсора.
 *     uint8_t  cntError=0;     //Счетчик ошибок
 *        
 *  2. Измерения на сенсоре выполняет функция dht.RT_run(pin). 
 *     При первом запуске сенсор, подключенный к пину=Pin  однократно инициализируется и устанавливает значения переменных
 *     xxx.permit=true
 *     xxx.start=true;
 *  3. Измерение проиcходит по двум управляющим сигналам: xxx.start=xxx.permit=true. 
 *  4. Функция xxx.RT_run(pin) возвращает значение переменной xxx.ok, которая равна false, если нет результатов измерения
 *     или  однократное true, если измерение выполнено. 
 *     После выполнения измерения, функция сбрасывает сигнал dht.start и для повторного запуска этот сигнал должен 
 *     быть повторно взведен внешней программой (обычно функцией генерируюшей короткие - на один цикл loop() - импульсы).
 *  5. Функция xxx.RT_run() обновляет результаты измерений только если не было ошибок измерения. 
 *     После повторяющейся подряд трехкратной (значение определяется переменной maxError<=7) ошибки измерения, 
 *     результатам измерения присваиваются значения: humidity = NAN; temperature = NAN;  
 *  6. Коды ошибок:=0 - нет ошибок       
 *      =1 - недопустимый номер пина (=99, присваивается конструктором и меняется при вызове функции init()
 *      =2 - ошибка по тайм-ауту
 *      =3 - ошибка по КС
 *      =4 - выход за диапазон измерений
 *  7. По конструктивным особенностям сенсора вызовы не должны бвть чаще чем 1 раз в 2 сек    
 *  8. Запущенный процесс измерения занимает примерно 5,7мс - в это время процессор занят и реагирует 
 *     только на аппаратные прерывания, время обработки которого не должен превышать 30-50мкс - 
 *     это может привести к потери сигнала с сенсора и соответственно приведет к ошибке измерения. 
 *     Вероятность подобной  трехкратной ошибки крайне низкая и потому не стоит принимать ее во внимание. 
 *      
 ***********************************************************
  Написана ecoins (ecoins@mail.ru) (использовались фрагменты  библиотеки Mark Ruys, mark@paracas.nl http://www.github.com/markruys/arduino-DHT
  Лист изменений
   2018-01-21: Начальная версия
   2018-08-12: Оптимизированая под работу  с библиотекой RT_Hardware
 ******************************************************************/
 
#ifndef RT_HW_DHTx_h
#define RT_HW_DHTx_h

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif


#define RT_HW_DHTx_VER 12
//-------------------------------------------------------------------------------------------------
//			             КЛАСС ДАННЫХ DHT22
//-------------------------------------------------------------------------------------------------
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_DHTx_ID{
public:
// ****** Данные ****** 2+2+4+4=12 байт
int16_t	 pin, numIRQ;			//--номер пина
uint32_t init: 1,initAdd:1,modeIRQ:1,extRUN:1, //
		 run:  1, runIRQ:1,fresh:  1, 
		 start:1,begin:  1,end:    1,
		 ok:   1,error:  1,codeErr:3,cntErr: 3,maxErr:3, //--флаг ошибки,код ошибки,счетчик ошибок при чтении(сбрасывается при ок),макс.кол-во ошибок(<=7)
		 startOk:1, meterOk:1, convertOk:1;	
uint16_t stepStart:2, stepMeter:3, stepNoIRQ:3, stepYesIRQ:3, stepConvert:3;		 
uint16_t cntMeter;	
uint16_t rawHumidity, rawTemperature, data;	 		
				;
//uint32_t state:4,
//		 ok:1,          //--результат измерение (= true - завершено)    
//		 error:1,		//--измерение завершено с ошибкой(=true)
		 //init:1,		//--флаг инициализации сенсора
//		 error:1,codeErr:1,cntErr:3, maxErr:3, //--флаг ошибки,код ошибки,счетчик ошибок при чтении(сбрасывается при ок),макс.кол-во ошибок(<=7)
	
//		 step:3, stepIRQ:3;
float    temperature;	//--температура
float    humidity;		//--влажность	
uint32_t period, 		//--период опроса
         timeStartDelay, 
		 timePeriod,    //--рабочий регистр
		 timeFLProg;	//
// ****** Методы ******
RT_HW_DHTx_ID();    //--Конструктор
void IRQ_handler(); //--Обработчик прерываний
};
#pragma pack(pop)				//---Восстановление предыдущей настройки
//++++++++++++++++++++Описание класса DHT_RT++++++++++++++++++++++++++++
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
class RT_HW_DHTx{
public:
// ****** Данные ******
// ****** Методы ******
RT_HW_DHTx(){}; 											//--Конструктор
bool    setID 		(RT_HW_DHTx_ID &id, int16_t pin=-1);    //--Инициализация сенсора                                     
bool    meter 		(RT_HW_DHTx_ID &id);					//--Запуск в режиме реального времени   
bool    meterStart	(RT_HW_DHTx_ID &id);					//--Запуск сенсора на подговку к измерению
bool    meterNoIRQ	(RT_HW_DHTx_ID &id);
bool    meterYesIRQ	(RT_HW_DHTx_ID &id);
bool    convert     (RT_HW_DHTx_ID &id);
//--------------------------------------------------------------------------------------------------
uint32_t getPastTense	(uint32_t &t1, uint32_t t2);		//--ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ 
bool 	 eventMillis	(uint32_t &t1, uint32_t val);	    //--ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
bool 	 periodMillis   (uint8_t &s,   uint32_t &t1, uint32_t v);
void 	 setPeriod(RT_HW_DHTx_ID &id, int32_t extPeriod); 
};  //---END 
#pragma pack(pop)				//---Восстановление предыдущей настройки
//-------------------------------------------------------------
//==============  Конец класса ======================================================
extern RT_HW_DHTx RT_HW_dhtX; //--Создание объекта LCD типа HD44780
//===================================================================================
#endif //---RT_HW_DHTx_h


